@@ -8,17 +8,21 @@ builddir=$1
 setup=$2
 suite=$3
 multiplier=${MESON_TEST_TIMEOUT_MULTIPLIER:-1}
+n_processes=${MESON_TEST_MAX_PROCESSES:-$(nproc)}
 
 # Ignore memory leaks lower in dependencies
-export LSAN_OPTIONS=suppressions=$srcdir/lsan.supp:print_suppressions=0:detect_leaks=0:allocator_may_return_null=1
-export G_SLICE=always-malloc
+export LSAN_OPTIONS=suppressions=$srcdir/lsan.supp:print_suppressions=0:detect_leaks=0:allocator_may_return_null=1:symbolize=1
+export ASAN_SYMBOLIZER_PATH=/usr/bin/llvm-symbolizer
+
 
 case "${setup}" in
   x11*)
-    xvfb-run -a -s "-screen 0 1024x768x24 -noreset" \
+    dbus-run-session -- \
+       xvfb-run -a -s "-screen 0 1024x768x24 -noreset" \
           meson test -C ${builddir} \
                 --quiet \
                 --timeout-multiplier "${multiplier}" \
+                --num-processes "${n_processes}" \
                 --print-errorlogs \
                 --setup=${setup} \
                 --suite=${suite//,/ --suite=} \
@@ -40,9 +44,11 @@ case "${setup}" in
     compositor=$!
     export WAYLAND_DISPLAY=wayland-5
 
-    meson test -C ${builddir} \
+    dbus-run-session -- \
+          meson test -C ${builddir} \
                 --quiet \
                 --timeout-multiplier "${multiplier}" \
+                --num-processes "${n_processes}" \
                 --print-errorlogs \
                 --setup=${setup} \
                 --suite=${suite//,/ --suite=} \
@@ -63,9 +69,11 @@ case "${setup}" in
     server=$!
     export BROADWAY_DISPLAY=:5
 
-    meson test -C ${builddir} \
+    dbus-run-session -- \
+          meson test -C ${builddir} \
                 --quiet \
                 --timeout-multiplier "${multiplier}" \
+                --num-processes "${n_processes}" \
                 --print-errorlogs \
                 --setup=${setup} \
                 --suite=${suite//,/ --suite=} \
@@ -87,13 +95,6 @@ esac
 
 cd ${builddir}
 
-$srcdir/.gitlab-ci/meson-junit-report.py \
-            --project-name=gtk \
-            --backend="${setup}" \
-            --job-id="${CI_JOB_NAME}" \
-            --output="report-${setup}.xml" \
-            "meson-logs/testlog-${setup}.json"
-
 $srcdir/.gitlab-ci/meson-html-report.py \
             --project-name=gtk \
             --backend="${setup}" \
