@@ -600,8 +600,8 @@ update_display (void)
   if (s->len > 0)
     {
       pango_font_description_set_variations (desc, s->str);
-      g_string_free (s, TRUE);
     }
+  g_string_free (s, TRUE);
 
   font_desc = pango_font_description_to_string (desc);
 
@@ -811,7 +811,7 @@ tag_pair_equal (gconstpointer a, gconstpointer b)
 }
 
 
-static GtkOrdering
+static int
 script_sort (const void *item1,
              const void *item2,
              void       *data)
@@ -1586,7 +1586,7 @@ update_font_variations (void)
         }
 
       gtk_grid_attach (GTK_GRID (demo->variations_grid), combo, 1, -1, 3, 1);
-      g_signal_connect (combo, "notify::selecte", G_CALLBACK (instance_changed), NULL);
+      g_signal_connect (combo, "notify::selected", G_CALLBACK (instance_changed), NULL);
       demo->instance_combo = combo;
    }
 
@@ -1869,4 +1869,4 @@ do_font_features (GtkWidget *do_widget)
   return window;
 }
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
