@@ -257,12 +257,6 @@ gdk_broadway_display_flush (GdkDisplay *display)
   _gdk_broadway_server_flush (broadway_display->server);
 }
 
-static gboolean
-gdk_broadway_display_has_pending (GdkDisplay *display)
-{
-  return FALSE;
-}
-
 static void
 gdk_broadway_display_dispose (GObject *object)
 {
@@ -477,18 +471,18 @@ gdk_broadway_display_class_init (GdkBroadwayDisplayClass * class)
   object_class->dispose = gdk_broadway_display_dispose;
   object_class->finalize = gdk_broadway_display_finalize;
 
+  display_class->toplevel_type = GDK_TYPE_BROADWAY_TOPLEVEL;
+  display_class->popup_type = GDK_TYPE_BROADWAY_POPUP;
   display_class->cairo_context_type = GDK_TYPE_BROADWAY_CAIRO_CONTEXT;
 
   display_class->get_name = gdk_broadway_display_get_name;
   display_class->beep = gdk_broadway_display_beep;
   display_class->sync = gdk_broadway_display_sync;
   display_class->flush = gdk_broadway_display_flush;
-  display_class->has_pending = gdk_broadway_display_has_pending;
   display_class->queue_events = _gdk_broadway_display_queue_events;
 
   display_class->get_next_serial = gdk_broadway_display_get_next_serial;
   display_class->notify_startup_complete = gdk_broadway_display_notify_startup_complete;
-  display_class->create_surface = _gdk_broadway_display_create_surface;
   display_class->get_keymap = _gdk_broadway_display_get_keymap;
 
   display_class->get_monitors = gdk_broadway_display_get_monitors;
