@@ -230,7 +230,7 @@ gdk_display_class_init (GdkDisplayClass *class)
   class->opened = gdk_display_real_opened;
 
   /**
-   * GdkDisplay:composited: (attributes org.gtk.Property.get=gdk_display_is_composited)
+   * GdkDisplay:composited: (getter is_composited)
    *
    * %TRUE if the display properly composites the alpha channel.
    */
@@ -240,7 +240,7 @@ gdk_display_class_init (GdkDisplayClass *class)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDisplay:rgba: (attributes org.gtk.Property.get=gdk_display_is_rgba)
+   * GdkDisplay:rgba: (getter is_rgba)
    *
    * %TRUE if the display supports an alpha channel.
    */
@@ -250,7 +250,7 @@ gdk_display_class_init (GdkDisplayClass *class)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDisplay:shadow-width: (attributes org.gtk.Property.get=gdk_display_supports_shadow_width)
+   * GdkDisplay:shadow-width: (getter supports_shadow_width)
    *
    * %TRUE if the display supports extensible frames.
    *
@@ -262,7 +262,7 @@ gdk_display_class_init (GdkDisplayClass *class)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDisplay:input-shapes: (attributes org.gtk.Property.get=gdk_display_supports_input_shapes)
+   * GdkDisplay:input-shapes: (getter supports_input_shapes)
    *
    * %TRUE if the display supports input shapes.
    */
@@ -272,7 +272,7 @@ gdk_display_class_init (GdkDisplayClass *class)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDisplay:dmabuf-formats: (attributes org.gtk.Property.get=gdk_display_get_dmabuf_formats)
+   * GdkDisplay:dmabuf-formats:
    *
    * The dma-buf formats that are supported on this display
    *
@@ -418,15 +418,17 @@ gdk_display_dispose (GObject *object)
 {
   GdkDisplay *display = GDK_DISPLAY (object);
   GdkDisplayPrivate *priv = gdk_display_get_instance_private (display);
-  gsize i;
 
-  for (i = 0; i < G_N_ELEMENTS (display->dmabuf_downloaders); i++)
+  if (display->vk_downloader)
     {
-      if (display->dmabuf_downloaders[i] == NULL)
-        continue;
+      gdk_dmabuf_downloader_close (display->vk_downloader);
+      g_clear_object (&display->vk_downloader);
+    }
 
-      gdk_dmabuf_downloader_close (display->dmabuf_downloaders[i]);
-      g_clear_object (&display->dmabuf_downloaders[i]);
+  if (display->egl_downloader)
+    {
+      gdk_dmabuf_downloader_close (display->egl_downloader);
+      g_clear_object (&display->egl_downloader);
     }
 
   _gdk_display_manager_remove_display (gdk_display_manager_get (), display);
@@ -434,13 +436,12 @@ gdk_display_dispose (GObject *object)
   g_queue_clear (&display->queued_events);
 
   g_clear_pointer (&display->egl_dmabuf_formats, gdk_dmabuf_formats_unref);
-  g_clear_pointer (&display->egl_external_formats, gdk_dmabuf_formats_unref);
+  g_clear_pointer (&display->egl_internal_formats, gdk_dmabuf_formats_unref);
 #ifdef GDK_RENDERING_VULKAN
-  if (display->vk_dmabuf_formats)
-    {
-      gdk_display_unref_vulkan (display);
-      g_assert (display->vk_dmabuf_formats == NULL);
-    }
+  if (display->vk_instance)
+    gdk_display_destroy_vulkan_instance (display);
+  g_assert (display->vk_dmabuf_formats == NULL);
+  g_clear_error (&display->vulkan_error);
 #endif
 
   g_clear_object (&priv->gl_context);
@@ -1116,7 +1117,7 @@ gdk_display_get_primary_clipboard (GdkDisplay *display)
 }
 
 /**
- * gdk_display_supports_input_shapes: (attributes org.gtk.Method.get_property=input-shapes)
+ * gdk_display_supports_input_shapes: (get-property input-shapes)
  * @display: a `GdkDisplay`
  *
  * Returns %TRUE if the display supports input shapes.
@@ -1196,8 +1197,9 @@ gdk_display_get_app_launch_context (GdkDisplay *display)
 GdkDisplay *
 gdk_display_open (const char *display_name)
 {
-  return gdk_display_manager_open_display (gdk_display_manager_get (),
-                                           display_name);
+  gdk_ensure_initialized ();
+
+  return gdk_display_manager_open_display (gdk_display_manager_get (), display_name);
 }
 
 gulong
@@ -1283,6 +1285,53 @@ gdk_display_get_keymap (GdkDisplay *display)
   return GDK_DISPLAY_GET_CLASS (display)->get_keymap (display);
 }
 
+/*< private >
+ * gdk_display_prepare_vulkan:
+ * @self: a `GdkDisplay`
+ * @error: return location for a `GError`
+ *
+ * Checks that Vulkan is available for @self and ensures that it is
+ * properly initialized.
+ *
+ * When this fails, an @error will be set describing the error and this
+ * function returns %FALSE.
+ *
+ * Note that even if this function succeeds, creating a `GdkVulkanContext`
+ * may still fail.
+ *
+ * This function is idempotent. Calling it multiple times will just
+ * return the same value or error.
+ *
+ * You never need to call this function, GDK will call it automatically
+ * as needed.
+ *
+ * Returns: %TRUE if the display supports Vulkan
+ */
+gboolean
+gdk_display_prepare_vulkan (GdkDisplay  *self,
+                            GError     **error)
+{
+  g_return_val_if_fail (GDK_IS_DISPLAY (self), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+#ifdef GDK_RENDERING_VULKAN
+  if (!self->vk_instance && !self->vulkan_error)
+    gdk_display_create_vulkan_instance (self, &self->vulkan_error);
+
+  if (self->vk_instance == NULL)
+    {
+      if (error)
+        *error = g_error_copy (self->vulkan_error);
+    }
+
+  return self->vk_instance != NULL;
+#else
+  g_set_error (error, GDK_VULKAN_ERROR, GDK_VULKAN_ERROR_UNSUPPORTED,
+               "GTK was built without Vulkan support");
+  return FALSE;
+#endif
+}
+
 /*<private>
  * gdk_display_create_vulkan_context:
  * @self: a `GdkDisplay`
@@ -1309,10 +1358,10 @@ gdk_display_create_vulkan_context (GdkDisplay  *self,
   g_return_val_if_fail (surface == NULL || GDK_IS_SURFACE (surface), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  if (gdk_display_get_debug_flags (self) & GDK_DEBUG_VULKAN_DISABLE)
+  if (!gdk_has_feature (GDK_FEATURE_VULKAN))
     {
       g_set_error_literal (error, GDK_VULKAN_ERROR, GDK_VULKAN_ERROR_NOT_AVAILABLE,
-                           _("Vulkan support disabled via GDK_DEBUG"));
+                           _("Vulkan support disabled via GDK_DISABLE"));
       return NULL;
     }
 
@@ -1362,11 +1411,11 @@ gdk_display_init_gl (GdkDisplay *self)
 
   before = GDK_PROFILER_CURRENT_TIME;
 
-  if (gdk_display_get_debug_flags (self) & GDK_DEBUG_GL_DISABLE)
+  if (!gdk_has_feature (GDK_FEATURE_OPENGL))
     {
       g_set_error_literal (&priv->gl_error, GDK_GL_ERROR,
                            GDK_GL_ERROR_NOT_AVAILABLE,
-                           _("GL support disabled via GDK_DEBUG"));
+                           _("OpenGL support disabled via GDK_DISABLE"));
       return;
     }
 
@@ -1477,7 +1526,7 @@ gdk_display_create_gl_context (GdkDisplay  *self,
   if (!gdk_display_prepare_gl (self, error))
     return NULL;
 
-  return gdk_gl_context_new (self, NULL);
+  return gdk_gl_context_new (self, NULL, FALSE);
 }
 
 /*< private >
@@ -1570,19 +1619,27 @@ describe_egl_config (EGLDisplay egl_display,
 }
 
 gpointer
-gdk_display_get_egl_config (GdkDisplay *self)
+gdk_display_get_egl_config (GdkDisplay     *self,
+                            GdkMemoryDepth  depth)
 {
   GdkDisplayPrivate *priv = gdk_display_get_instance_private (self);
 
-  return priv->egl_config;
-}
+  switch (depth)
+    {
+      case GDK_MEMORY_NONE:
+      case GDK_MEMORY_U8:
+      case GDK_MEMORY_U8_SRGB:
+        return priv->egl_config;
 
-gpointer
-gdk_display_get_egl_config_high_depth (GdkDisplay *self)
-{
-  GdkDisplayPrivate *priv = gdk_display_get_instance_private (self);
+      case GDK_MEMORY_U16:
+      case GDK_MEMORY_FLOAT16:
+      case GDK_MEMORY_FLOAT32:
+        return priv->egl_config_high_depth;
 
-  return priv->egl_config_high_depth;
+      case GDK_N_DEPTHS:
+      default:
+        g_return_val_if_reached (priv->egl_config);
+    }
 }
 
 static EGLDisplay
@@ -1874,6 +1931,8 @@ gdk_display_init_egl (GdkDisplay  *self,
     epoxy_has_egl_extension (priv->egl_display, "EGL_EXT_image_dma_buf_import_modifiers");
   self->have_egl_dma_buf_export =
     epoxy_has_egl_extension (priv->egl_display, "EGL_MESA_image_dma_buf_export");
+  self->have_egl_gl_colorspace =
+    epoxy_has_egl_extension (priv->egl_display, "EGL_KHR_gl_colorspace");
 
   if (self->have_egl_no_config_context)
     priv->egl_config_high_depth = gdk_display_create_egl_config (self,
@@ -1955,33 +2014,6 @@ gdk_display_get_egl_display (GdkDisplay *self)
 #endif
 }
 
-#ifdef HAVE_DMABUF
-static void
-gdk_display_add_dmabuf_downloader (GdkDisplay          *display,
-                                   GdkDmabufDownloader *downloader)
-{
-  gsize i;
-
-  if (downloader == NULL)
-    return;
-
-  /* dmabuf_downloaders is NULL-terminated */
-  for (i = 0; i < G_N_ELEMENTS (display->dmabuf_downloaders) - 1; i++)
-    {
-      if (display->dmabuf_downloaders[i] == NULL)
-        break;
-    }
-
-  g_assert (i < G_N_ELEMENTS (display->dmabuf_downloaders) - 1);
-
-  display->dmabuf_downloaders[i] = downloader;
-}
-#endif
-
-/* To support a drm format, we must be able to import it into GL
- * using the relevant EGL extensions, and download it into a memory
- * texture, possibly doing format conversion with shaders (in GSK).
- */
 void
 gdk_display_init_dmabuf (GdkDisplay *self)
 {
@@ -1996,25 +2028,28 @@ gdk_display_init_dmabuf (GdkDisplay *self)
   builder = gdk_dmabuf_formats_builder_new ();
 
 #ifdef HAVE_DMABUF
-  if (!GDK_DISPLAY_DEBUG_CHECK (self, DMABUF_DISABLE))
+  if (gdk_has_feature (GDK_FEATURE_DMABUF))
     {
 #ifdef GDK_RENDERING_VULKAN
-      gdk_display_add_dmabuf_downloader (self, gdk_vulkan_get_dmabuf_downloader (self, builder));
+      gdk_vulkan_init_dmabuf (self);
+      if (self->vk_dmabuf_formats)
+        gdk_dmabuf_formats_builder_add_formats (builder, self->vk_dmabuf_formats);
 #endif
 
 #ifdef HAVE_EGL
-      gdk_display_add_dmabuf_downloader (self, gdk_dmabuf_get_egl_downloader (self, builder));
+      gdk_dmabuf_egl_init (self);
+      if (self->egl_dmabuf_formats)
+        gdk_dmabuf_formats_builder_add_formats (builder, self->egl_dmabuf_formats);
 #endif
 
-      gdk_dmabuf_formats_builder_add_formats (builder,
-                                              gdk_dmabuf_get_mmap_formats ());
+      gdk_dmabuf_formats_builder_add_formats (builder, gdk_dmabuf_get_mmap_formats ());
     }
 #endif
 
   self->dmabuf_formats = gdk_dmabuf_formats_builder_free_to_formats (builder);
 
   GDK_DISPLAY_DEBUG (self, DMABUF,
-                     "Initialized support for %zu dmabuf formats",
+                     "Initialization finished. Advertising %zu dmabuf formats",
                      gdk_dmabuf_formats_get_n_formats (self->dmabuf_formats));
 }
 
@@ -2065,7 +2100,7 @@ gdk_display_set_debug_flags (GdkDisplay    *display,
 }
 
 /**
- * gdk_display_is_composited: (attributes org.gtk.Method.get_property=composited)
+ * gdk_display_is_composited: (get-property composited)
  * @display: a `GdkDisplay`
  *
  * Returns whether surfaces can reasonably be expected to have
@@ -2110,7 +2145,7 @@ gdk_display_set_composited (GdkDisplay *display,
 }
 
 /**
- * gdk_display_is_rgba: (attributes org.gtk.Method.get_property=rgba)
+ * gdk_display_is_rgba: (get-property rgba)
  * @display: a `GdkDisplay`
  *
  * Returns whether surfaces on this @display are created with an
@@ -2155,7 +2190,7 @@ gdk_display_set_rgba (GdkDisplay *display,
 }
 
 /**
- * gdk_display_supports_shadow_width: (attributes org.gtk.Method.get_property=shadow-width)
+ * gdk_display_supports_shadow_width: (get-property shadow-width)
  * @display: a `GdkDisplay`
  *
  * Returns whether it's possible for a surface to draw outside of the window area.
