@@ -24,7 +24,9 @@
 #include "gdkdebugprivate.h"
 #include "gdkdisplay.h"
 #include "gdkenumtypes.h"
+#include "gdkcolorstate.h"
 #include "gdkdmabuftextureprivate.h"
+#include "gdkdmabuftexturebuilderprivate.h"
 
 #include <cairo-gobject.h>
 
@@ -40,6 +42,8 @@ struct _GdkDmabufTextureBuilder
 
   GdkDmabuf dmabuf;
 
+  GdkColorState *color_state;
+
   GdkTexture *update_texture;
   cairo_region_t *update_region;
 };
@@ -123,6 +127,7 @@ enum
   PROP_MODIFIER,
   PROP_PREMULTIPLIED,
   PROP_N_PLANES,
+  PROP_COLOR_STATE,
   PROP_UPDATE_REGION,
   PROP_UPDATE_TEXTURE,
 
@@ -140,6 +145,7 @@ gdk_dmabuf_texture_builder_dispose (GObject *object)
 
   g_clear_object (&self->update_texture);
   g_clear_pointer (&self->update_region, cairo_region_destroy);
+  g_clear_pointer (&self->color_state, gdk_color_state_unref);
 
   G_OBJECT_CLASS (gdk_dmabuf_texture_builder_parent_class)->dispose (object);
 }
@@ -182,6 +188,10 @@ gdk_dmabuf_texture_builder_get_property (GObject    *object,
       g_value_set_uint (value, self->dmabuf.n_planes);
       break;
 
+    case PROP_COLOR_STATE:
+      g_value_set_boxed (value, self->color_state);
+      break;
+
     case PROP_UPDATE_REGION:
       g_value_set_boxed (value, self->update_region);
       break;
@@ -234,6 +244,10 @@ gdk_dmabuf_texture_builder_set_property (GObject      *object,
       gdk_dmabuf_texture_builder_set_n_planes (self, g_value_get_uint (value));
       break;
 
+    case PROP_COLOR_STATE:
+      gdk_dmabuf_texture_builder_set_color_state (self, g_value_get_boxed (value));
+      break;
+
     case PROP_UPDATE_REGION:
       gdk_dmabuf_texture_builder_set_update_region (self, g_value_get_boxed (value));
       break;
@@ -258,7 +272,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
   gobject_class->set_property = gdk_dmabuf_texture_builder_set_property;
 
   /**
-   * GdkDmabufTextureBuilder:display: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_display org.gtk.Property.set=gdk_dmabuf_texture_builder_set_display)
+   * GdkDmabufTextureBuilder:display:
    *
    * The display that this texture will be used on.
    *
@@ -270,7 +284,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:width: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_width org.gtk.Property.set=gdk_dmabuf_texture_builder_set_width)
+   * GdkDmabufTextureBuilder:width:
    *
    * The width of the texture.
    *
@@ -282,7 +296,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:height: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_height org.gtk.Property.set=gdk_dmabuf_texture_builder_set_height)
+   * GdkDmabufTextureBuilder:height:
    *
    * The height of the texture.
    *
@@ -294,7 +308,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:fourcc: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_fourcc org.gtk.Property.set=gdk_dmabuf_texture_builder_set_fourcc)
+   * GdkDmabufTextureBuilder:fourcc:
    *
    * The format of the texture, as a fourcc value.
    *
@@ -332,7 +346,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:n-planes: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_n_planes org.gtk.Property.set=gdk_dmabuf_texture_builder_set_n_planes)
+   * GdkDmabufTextureBuilder:n-planes:
    *
    * The number of planes of the texture.
    *
@@ -347,9 +361,21 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:update-region: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_update_region org.gtk.Property.set=gdk_dmabuf_texture_builder_set_update_region)
+   * GdkDmabufTextureBuilder:color-state:
    *
-   * The update region for [property@Gdk.GLTextureBuilder:update-texture].
+   * The color state of the texture.
+   *
+   * Since: 4.16
+   */
+  properties[PROP_COLOR_STATE] =
+    g_param_spec_boxed ("color-state", NULL, NULL,
+                        GDK_TYPE_COLOR_STATE,
+                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GdkDmabufTextureBuilder:update-region:
+   *
+   * The update region for [property@Gdk.DmabufTextureBuilder:update-texture].
    *
    * Since: 4.14
    */
@@ -359,7 +385,7 @@ gdk_dmabuf_texture_builder_class_init (GdkDmabufTextureBuilderClass *klass)
                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDmabufTextureBuilder:update-texture: (attributes org.gtk.Property.get=gdk_dmabuf_texture_builder_get_update_texture org.gtk.Property.set=gdk_dmabuf_texture_builder_set_update_texture)
+   * GdkDmabufTextureBuilder:update-texture:
    *
    * The texture [property@Gdk.DmabufTextureBuilder:update-region] is an update for.
    *
@@ -382,6 +408,8 @@ gdk_dmabuf_texture_builder_init (GdkDmabufTextureBuilder *self)
 
   for (int i = 0; i < GDK_DMABUF_MAX_PLANES; i++)
     self->dmabuf.planes[i].fd = -1;
+
+  self->color_state = NULL;
 }
 
 /**
@@ -443,7 +471,7 @@ gdk_dmabuf_texture_builder_set_display (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_width: (attributes org.gtk.Method.get_property=width)
+ * gdk_dmabuf_texture_builder_get_width:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the width previously set via gdk_dmabuf_texture_builder_set_width() or
@@ -462,13 +490,13 @@ gdk_dmabuf_texture_builder_get_width (GdkDmabufTextureBuilder *self)
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_width: (attributes org.gtk.Method.set_property=width)
+ * gdk_dmabuf_texture_builder_set_width:
  * @self: a `GdkDmabufTextureBuilder`
  * @width: The texture's width or 0 to unset
  *
  * Sets the width of the texture.
  *
- * The width must be set before calling [method@Gdk.GLTextureBuilder.build].
+ * The width must be set before calling [method@Gdk.DmabufTextureBuilder.build].
  *
  * Since: 4.14
  */
@@ -487,7 +515,7 @@ gdk_dmabuf_texture_builder_set_width (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_height: (attributes org.gtk.Method.get_property=height)
+ * gdk_dmabuf_texture_builder_get_height:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the height previously set via gdk_dmabuf_texture_builder_set_height() or
@@ -506,13 +534,13 @@ gdk_dmabuf_texture_builder_get_height (GdkDmabufTextureBuilder *self)
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_height: (attributes org.gtk.Method.set_property=height)
+ * gdk_dmabuf_texture_builder_set_height:
  * @self: a `GdkDmabufTextureBuilder`
  * @height: the texture's height or 0 to unset
  *
  * Sets the height of the texture.
  *
- * The height must be set before calling [method@Gdk.GLTextureBuilder.build].
+ * The height must be set before calling [method@Gdk.DmabufTextureBuilder.build].
  *
  * Since: 4.14
  */
@@ -531,7 +559,7 @@ gdk_dmabuf_texture_builder_set_height (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_fourcc: (attributes org.gtk.Method.get_property=fourcc)
+ * gdk_dmabuf_texture_builder_get_fourcc:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the format previously set via gdk_dmabuf_texture_builder_set_fourcc()
@@ -552,7 +580,7 @@ gdk_dmabuf_texture_builder_get_fourcc (GdkDmabufTextureBuilder *self)
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_fourcc: (attributes org.gtk.Method.set_property=fourcc)
+ * gdk_dmabuf_texture_builder_set_fourcc:
  * @self: a `GdkDmabufTextureBuilder`
  * @fourcc: the texture's format or 0 to unset
  *
@@ -560,7 +588,7 @@ gdk_dmabuf_texture_builder_get_fourcc (GdkDmabufTextureBuilder *self)
  *
  * The format is specified as a fourcc code.
  *
- * The format must be set before calling [method@Gdk.GLTextureBuilder.build].
+ * The format must be set before calling [method@Gdk.DmabufTextureBuilder.build].
  *
  * Since: 4.14
  */
@@ -620,7 +648,7 @@ gdk_dmabuf_texture_builder_set_modifier (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_n_planes: (attributes org.gtk.Method.get_property=n-planes)
+ * gdk_dmabuf_texture_builder_get_n_planes:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the number of planes.
@@ -682,7 +710,7 @@ gdk_dmabuf_texture_builder_set_premultiplied (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_n_planes: (attributes org.gtk.Method.set_property=n-planes)
+ * gdk_dmabuf_texture_builder_set_n_planes:
  * @self: a `GdkDmabufTextureBuilder`
  * @n_planes: the number of planes
  *
@@ -779,7 +807,7 @@ gdk_dmabuf_texture_builder_get_stride (GdkDmabufTextureBuilder *self,
  *
  * Sets the stride for a plane.
  *
- * The stride must be set for all planes before calling [method@Gdk.GLTextureBuilder.build].
+ * The stride must be set for all planes before calling [method@Gdk.DmabufTextureBuilder.build].
  *
  * Since: 4.14
  */
@@ -843,7 +871,56 @@ gdk_dmabuf_texture_builder_set_offset (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_update_texture: (attributes org.gtk.Method.get_property=update-texture)
+ * gdk_dmabuf_texture_builder_get_color_state:
+ * @self: a `GdkDmabufTextureBuilder`
+ *
+ * Gets the color state previously set via gdk_dmabuf_texture_builder_set_color_state().
+ *
+ * Returns: (nullable) (transfer none): the color state
+ *
+ * Since: 4.16
+ */
+GdkColorState *
+gdk_dmabuf_texture_builder_get_color_state (GdkDmabufTextureBuilder *self)
+{
+  g_return_val_if_fail (GDK_IS_DMABUF_TEXTURE_BUILDER (self), NULL);
+
+  return self->color_state;
+}
+
+/**
+ * gdk_dmabuf_texture_builder_set_color_state:
+ * @self: a `GdkDmabufTextureBuilder`
+ * @color_state: (nullable): a `GdkColorState` or `NULL` to unset the colorstate.
+ *
+ * Sets the color state for the texture.
+ *
+ * By default, the colorstate is `NULL`. In that case, GTK will choose the
+ * correct colorstate based on the format.
+ * If you don't know what colorstates are, this is probably the right thing.
+ *
+ * Since: 4.16
+ */
+void
+gdk_dmabuf_texture_builder_set_color_state (GdkDmabufTextureBuilder *self,
+                                            GdkColorState           *color_state)
+{
+  g_return_if_fail (GDK_IS_DMABUF_TEXTURE_BUILDER (self));
+
+  if (self->color_state == color_state ||
+      (self->color_state != NULL && color_state != NULL && gdk_color_state_equal (self->color_state, color_state)))
+    return;
+
+  g_clear_pointer (&self->color_state, gdk_color_state_unref);
+  self->color_state = color_state;
+  if (color_state)
+    gdk_color_state_ref (color_state);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COLOR_STATE]);
+}
+
+/**
+ * gdk_dmabuf_texture_builder_get_update_texture:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the texture previously set via gdk_dmabuf_texture_builder_set_update_texture() or
@@ -862,7 +939,7 @@ gdk_dmabuf_texture_builder_get_update_texture (GdkDmabufTextureBuilder *self)
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_update_texture: (attributes org.gtk.Method.set_property=update-texture)
+ * gdk_dmabuf_texture_builder_set_update_texture:
  * @self: a `GdkDmabufTextureBuilder`
  * @texture: (nullable): the texture to update
  *
@@ -885,7 +962,7 @@ gdk_dmabuf_texture_builder_set_update_texture (GdkDmabufTextureBuilder *self,
 }
 
 /**
- * gdk_dmabuf_texture_builder_get_update_region: (attributes org.gtk.Method.get_property=update-region)
+ * gdk_dmabuf_texture_builder_get_update_region:
  * @self: a `GdkDmabufTextureBuilder`
  *
  * Gets the region previously set via gdk_dmabuf_texture_builder_set_update_region() or
@@ -904,7 +981,7 @@ gdk_dmabuf_texture_builder_get_update_region (GdkDmabufTextureBuilder *self)
 }
 
 /**
- * gdk_dmabuf_texture_builder_set_update_region: (attributes org.gtk.Method.set_property=update-region)
+ * gdk_dmabuf_texture_builder_set_update_region:
  * @self: a `GdkDmabufTextureBuilder`
  * @region: (nullable): the region to update
  *
@@ -948,22 +1025,20 @@ gdk_dmabuf_texture_builder_set_update_region (GdkDmabufTextureBuilder *self,
  *
  * Builds a new `GdkTexture` with the values set up in the builder.
  *
- * It is a programming error to call this function if any mandatory
- * property has not been set.
+ * It is a programming error to call this function if any mandatory property has not been set.
  *
- * If the dmabuf is not supported by GTK, %NULL will be returned and @error will be set.
+ * Not all formats defined in the `drm_fourcc.h` header are supported. You can use
+ * [method@Gdk.Display.get_dmabuf_formats] to get a list of supported formats. If the
+ * format is not supported by GTK, %NULL will be returned and @error will be set.
  *
  * The `destroy` function gets called when the returned texture gets released.
  *
- * It is possible to call this function multiple times to create multiple textures,
- * possibly with changing properties in between.
- *
  * It is the responsibility of the caller to keep the file descriptors for the planes
  * open until the created texture is no longer used, and close them afterwards (possibly
  * using the @destroy notify).
  *
- * Not all formats defined in the `drm_fourcc.h` header are supported. You can use
- * [method@Gdk.Display.get_dmabuf_formats] to get a list of supported formats.
+ * It is possible to call this function multiple times to create multiple textures,
+ * possibly with changing properties in between.
  *
  * Returns: (transfer full) (nullable): a newly built `GdkTexture` or `NULL`
  *   if the format is not supported
@@ -988,11 +1063,11 @@ gdk_dmabuf_texture_builder_build (GdkDmabufTextureBuilder *self,
   for (i = 0; i < self->dmabuf.n_planes; i++)
     g_return_val_if_fail (self->dmabuf.planes[i].fd != -1, NULL);
 
-  if (GDK_DISPLAY_DEBUG_CHECK (self->display, DMABUF_DISABLE))
+  if (!gdk_has_feature (GDK_FEATURE_DMABUF))
     {
       g_set_error_literal (error,
                            GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_NOT_AVAILABLE,
-                           "dmabuf support disabled via GDK_DEBUG environment variable");
+                           "dmabuf support disabled via GDK_DISABLE environment variable");
       return NULL;
     }
 
@@ -1004,3 +1079,19 @@ gdk_dmabuf_texture_builder_get_dmabuf (GdkDmabufTextureBuilder *self)
 {
   return &self->dmabuf;
 }
+
+void
+gdk_dmabuf_texture_builder_set_dmabuf (GdkDmabufTextureBuilder *self,
+                                       const GdkDmabuf         *dmabuf)
+{
+  gdk_dmabuf_texture_builder_set_fourcc (self, dmabuf->fourcc);
+  gdk_dmabuf_texture_builder_set_modifier (self, dmabuf->modifier);
+  gdk_dmabuf_texture_builder_set_n_planes (self, dmabuf->n_planes);
+
+  for (unsigned int i = 0; i < dmabuf->n_planes; i++)
+    {
+      gdk_dmabuf_texture_builder_set_fd (self, i, dmabuf->planes[i].fd);
+      gdk_dmabuf_texture_builder_set_stride (self, i, dmabuf->planes[i].stride);
+      gdk_dmabuf_texture_builder_set_offset (self, i, dmabuf->planes[i].offset);
+    }
+}
