@@ -48,6 +48,8 @@ struct _GdkDrawContextPrivate {
   GdkSurface *surface;
 
   cairo_region_t *frame_region;
+  GdkColorState *color_state;
+  GdkMemoryDepth depth;
 };
 
 enum {
@@ -170,7 +172,7 @@ gdk_draw_context_class_init (GdkDrawContextClass *klass)
   klass->empty_frame = gdk_draw_context_default_empty_frame;
 
   /**
-   * GdkDrawContext:display: (attributes org.gtk.Property.get=gdk_draw_context_get_display)
+   * GdkDrawContext:display:
    *
    * The `GdkDisplay` used to create the `GdkDrawContext`.
    */
@@ -182,7 +184,7 @@ gdk_draw_context_class_init (GdkDrawContextClass *klass)
                          G_PARAM_STATIC_STRINGS);
 
   /**
-   * GdkDrawContext:surface: (attributes org.gtk.Property.get=gdk_draw_context_get_surface)
+   * GdkDrawContext:surface:
    *
    * The `GdkSurface` the context is bound to.
    */
@@ -201,8 +203,12 @@ static guint pixels_counter;
 static void
 gdk_draw_context_init (GdkDrawContext *self)
 {
+  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (self);
+
   if (pixels_counter == 0)
     pixels_counter = gdk_profiler_define_int_counter ("frame pixels", "Pixels drawn per frame");
+
+  priv->depth = GDK_N_DEPTHS;
 }
 
 /**
@@ -217,6 +223,9 @@ gdk_draw_context_init (GdkDrawContext *self)
  *
  * Returns: %TRUE if the context is between [method@Gdk.DrawContext.begin_frame]
  *   and [method@Gdk.DrawContext.end_frame] calls.
+ *
+ * Deprecated: 4.16: Drawing directly to the surface is no longer recommended.
+ *   Use `GskRenderNode` and `GskRenderer`.
  */
 gboolean
 gdk_draw_context_is_in_frame (GdkDrawContext *context)
@@ -242,7 +251,7 @@ gdk_draw_context_surface_resized (GdkDrawContext *context)
 }
 
 /**
- * gdk_draw_context_get_display: (attributes org.gtk.Method.get_property=display)
+ * gdk_draw_context_get_display:
  * @context: a `GdkDrawContext`
  *
  * Retrieves the `GdkDisplay` the @context is created for
@@ -260,7 +269,7 @@ gdk_draw_context_get_display (GdkDrawContext *context)
 }
 
 /**
- * gdk_draw_context_get_surface: (attributes org.gtk.Method.get_property=surface)
+ * gdk_draw_context_get_surface:
  * @context: a `GdkDrawContext`
  *
  * Retrieves the surface that @context is bound to.
@@ -307,6 +316,9 @@ gdk_draw_context_get_surface (GdkDrawContext *context)
  * gdk_draw_context_begin_frame() and gdk_draw_context_end_frame() via the
  * use of [GskRenderer](../gsk4/class.Renderer.html)s, so application code
  * does not need to call these functions explicitly.
+ *
+ * Deprecated: 4.16: Drawing directly to the surface is no longer recommended.
+ *   Use `GskRenderNode` and `GskRenderer`.
  */
 void
 gdk_draw_context_begin_frame (GdkDrawContext       *context,
@@ -318,11 +330,12 @@ gdk_draw_context_begin_frame (GdkDrawContext       *context,
   g_return_if_fail (priv->surface != NULL);
   g_return_if_fail (region != NULL);
 
-  gdk_draw_context_begin_frame_full (context, GDK_MEMORY_U8, region);
+  gdk_draw_context_begin_frame_full (context, GDK_MEMORY_U8, region, NULL);
 }
 
 /*
  * @depth: best depth to render in
+ * @opaque: (nullable): opaque region of the rendering
  *
  * If the given depth is not `GDK_MEMORY_U8`, GDK will see about providing a
  * rendering target that supports a higher bit depth than 8 bits per channel.
@@ -345,9 +358,10 @@ gdk_draw_context_begin_frame (GdkDrawContext       *context,
  * to choose.
  */
 void
-gdk_draw_context_begin_frame_full (GdkDrawContext       *context,
-                                   GdkMemoryDepth        depth,
-                                   const cairo_region_t *region)
+gdk_draw_context_begin_frame_full (GdkDrawContext        *context,
+                                   GdkMemoryDepth         depth,
+                                   const cairo_region_t  *region,
+                                   const graphene_rect_t *opaque)
 {
   GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (context);
 
@@ -372,13 +386,25 @@ gdk_draw_context_begin_frame_full (GdkDrawContext       *context,
       return;
     }
 
+  gdk_surface_set_opaque_rect (priv->surface, opaque);
+
   if (gdk_display_get_debug_flags (priv->display) & GDK_DEBUG_HIGH_DEPTH)
     depth = GDK_MEMORY_FLOAT32;
 
   priv->frame_region = cairo_region_copy (region);
   priv->surface->paint_context = g_object_ref (context);
 
-  GDK_DRAW_CONTEXT_GET_CLASS (context)->begin_frame (context, depth, priv->frame_region);
+  g_assert (priv->color_state == NULL);
+
+  GDK_DRAW_CONTEXT_GET_CLASS (context)->begin_frame (context,
+                                                     depth,
+                                                     priv->frame_region,
+                                                     &priv->color_state,
+                                                     &priv->depth);
+
+  /* The callback is meant to set them. Note that it does not return a ref */
+  g_assert (priv->color_state != NULL);
+  g_assert (priv->depth < GDK_N_DEPTHS);
 
   cairo_region_intersect_rectangle (priv->frame_region,
                                     &(cairo_rectangle_int_t) {
@@ -406,6 +432,21 @@ region_get_pixels (cairo_region_t *region)
 }
 #endif
 
+void
+gdk_draw_context_end_frame_full (GdkDrawContext *context)
+{
+  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (context);
+
+  GDK_DRAW_CONTEXT_GET_CLASS (context)->end_frame (context, priv->frame_region);
+
+  gdk_profiler_set_int_counter (pixels_counter, region_get_pixels (priv->frame_region));
+
+  priv->color_state = NULL;
+  g_clear_pointer (&priv->frame_region, cairo_region_destroy);
+  g_clear_object (&priv->surface->paint_context);
+  priv->depth = GDK_N_DEPTHS;
+}
+
 /**
  * gdk_draw_context_end_frame:
  * @context: a `GdkDrawContext`
@@ -418,6 +459,9 @@ region_get_pixels (cairo_region_t *region)
  * When using a [class@Gdk.GLContext], this function may call `glFlush()`
  * implicitly before returning; it is not recommended to call `glFlush()`
  * explicitly before calling this function.
+ *
+ * Deprecated: 4.16: Drawing directly to the surface is no longer recommended.
+ *   Use `GskRenderNode` and `GskRenderer`.
  */
 void
 gdk_draw_context_end_frame (GdkDrawContext *context)
@@ -445,12 +489,7 @@ gdk_draw_context_end_frame (GdkDrawContext *context)
       return;
     }
 
-  GDK_DRAW_CONTEXT_GET_CLASS (context)->end_frame (context, priv->frame_region);
-
-  gdk_profiler_set_int_counter (pixels_counter, region_get_pixels (priv->frame_region));
-
-  g_clear_pointer (&priv->frame_region, cairo_region_destroy);
-  g_clear_object (&priv->surface->paint_context);
+  gdk_draw_context_end_frame_full (context);
 }
 
 /**
@@ -467,17 +506,58 @@ gdk_draw_context_end_frame (GdkDrawContext *context)
  * and [method@Gdk.DrawContext.end_frame], %NULL will be returned.
  *
  * Returns: (transfer none) (nullable): a Cairo region
+ *
+ * Deprecated: 4.16: Drawing directly to the surface is no longer recommended.
+ *   Use `GskRenderNode` and `GskRenderer`.
  */
 const cairo_region_t *
-gdk_draw_context_get_frame_region (GdkDrawContext *context)
+_gdk_draw_context_get_frame_region (GdkDrawContext *self)
 {
-  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (context);
-
-  g_return_val_if_fail (GDK_IS_DRAW_CONTEXT (context), NULL);
+  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (self);
 
   return priv->frame_region;
 }
 
+const cairo_region_t *
+(gdk_draw_context_get_frame_region) (GdkDrawContext *context)
+{
+  g_return_val_if_fail (GDK_IS_DRAW_CONTEXT (context), NULL);
+
+  return _gdk_draw_context_get_frame_region (context);
+}
+
+/*<private>
+ * gdk_draw_context_get_color_state:
+ * @self: a `GdkDrawContext`
+ *
+ * Gets the target color state while rendering. If no rendering is going on, %NULL is returned.
+ *
+ * Returns: (transfer none) (nullable): the target color state
+ **/
+GdkColorState *
+gdk_draw_context_get_color_state (GdkDrawContext *self)
+{
+  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (self);
+
+  return priv->color_state;
+}
+
+/*<private>
+ * gdk_draw_context_get_depth:
+ * @self: a `GdkDrawContext`
+ *
+ * Gets the target depth while rendering. If no rendering is going on, the return value is undefined.
+ *
+ * Returns: the target depth
+ **/
+GdkMemoryDepth
+gdk_draw_context_get_depth (GdkDrawContext *self)
+{
+  GdkDrawContextPrivate *priv = gdk_draw_context_get_instance_private (self);
+
+  return priv->depth;
+}
+
 void
 gdk_draw_context_empty_frame (GdkDrawContext *context)
 {
