@@ -978,9 +978,13 @@ gdk_event_get_axis (GdkEvent   *event,
  * according to platform conventions.
  *
  * The right mouse button typically triggers context menus.
+ * On macOS, Control+left mouse button also triggers.
  *
  * This function should always be used instead of simply checking for
- * event->button == %GDK_BUTTON_SECONDARY.
+ *
+ * ```c
+ * event->button == GDK_BUTTON_SECONDARY
+ * ```
  *
  * Returns: %TRUE if the event should trigger a context menu.
  */
@@ -998,6 +1002,13 @@ gdk_event_triggers_context_menu (GdkEvent *event)
       if (bevent->button == GDK_BUTTON_SECONDARY &&
           ! (bevent->state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK)))
         return TRUE;
+
+#ifdef __APPLE__
+      if (bevent->button == GDK_BUTTON_PRIMARY &&
+          (bevent->state & GDK_CONTROL_MASK) &&
+          ! (bevent->state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK)))
+        return TRUE;
+#endif
     }
 
   return FALSE;
@@ -1371,6 +1382,8 @@ gdk_event_get_modifier_state (GdkEvent *event)
  * Extract the event surface relative x/y coordinates from an event.
  *
  * This position is in [surface coordinates](coordinates.html).
+ *
+ * Returns: whether the positions were set
  */
 gboolean
 gdk_event_get_position (GdkEvent *event,
@@ -3431,4 +3444,4 @@ gdk_grab_broken_event_get_implicit (GdkEvent *event)
 
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
