@@ -50,14 +50,18 @@ struct _GdkVulkanContextClass
   GdkDrawContextClass parent_class;
 
 #ifdef GDK_RENDERING_VULKAN
-  VkResult     (* create_surface)       (GdkVulkanContext       *context,
-                                         VkSurfaceKHR           *surface);
+  VkResult              (* create_surface)                              (GdkVulkanContext      *context,
+                                                                         VkSurfaceKHR          *surface);
+  void                  (* get_image_size)                              (GdkVulkanContext      *context,
+                                                                         uint32_t              *out_width,
+                                                                         uint32_t              *out_height);
+
 #endif
 };
 
 #ifdef GDK_RENDERING_VULKAN
 
-const char *            gdk_vulkan_strerror                         (VkResult           result);
+const char *            gdk_vulkan_strerror                             (VkResult               result);
 
 static inline VkResult
 gdk_vulkan_handle_result (VkResult    res,
@@ -65,7 +69,7 @@ gdk_vulkan_handle_result (VkResult    res,
 {
   if (res != VK_SUCCESS)
     {
-      GDK_DEBUG (VULKAN, "%s(): %s (%d)", called_function, gdk_vulkan_strerror (res), res);
+      g_warning ("%s(): %s (%d)", called_function, gdk_vulkan_strerror (res), res);
     }
 
   return res;
@@ -73,15 +77,11 @@ gdk_vulkan_handle_result (VkResult    res,
 
 #define GDK_VK_CHECK(func, ...) gdk_vulkan_handle_result (func (__VA_ARGS__), G_STRINGIFY (func))
 
-gboolean                gdk_display_init_vulkan                         (GdkDisplay            *display,
+gboolean                gdk_display_create_vulkan_instance              (GdkDisplay            *display,
                                                                          GError               **error);
-void                    gdk_display_ref_vulkan                          (GdkDisplay            *display);
-void                    gdk_display_unref_vulkan                        (GdkDisplay            *display);
+void                    gdk_display_destroy_vulkan_instance             (GdkDisplay            *display);
 
-#ifdef HAVE_DMABUF
-GdkDmabufDownloader *   gdk_vulkan_get_dmabuf_downloader                (GdkDisplay            *display,
-                                                                         GdkDmabufFormatsBuilder *builder);
-#endif
+void                    gdk_vulkan_init_dmabuf                          (GdkDisplay            *display);
 
 VkShaderModule          gdk_display_get_vk_shader_module                (GdkDisplay            *display,
                                                                          const char            *resource_name);
@@ -94,14 +94,14 @@ VkDevice                gdk_vulkan_context_get_device                   (GdkVulk
 VkQueue                 gdk_vulkan_context_get_queue                    (GdkVulkanContext      *context);
 uint32_t                gdk_vulkan_context_get_queue_family_index       (GdkVulkanContext      *context);
 VkFormat                gdk_vulkan_context_get_image_format             (GdkVulkanContext      *context);
+GdkMemoryFormat         gdk_vulkan_context_get_memory_format            (GdkVulkanContext      *context);
 uint32_t                gdk_vulkan_context_get_n_images                 (GdkVulkanContext      *context);
 VkImage                 gdk_vulkan_context_get_image                    (GdkVulkanContext      *context,
                                                                          guint                  id);
 uint32_t                gdk_vulkan_context_get_draw_index               (GdkVulkanContext      *context);
-VkSemaphore             gdk_vulkan_context_get_draw_semaphore           (GdkVulkanContext      *context);
 
-GdkMemoryFormat         gdk_vulkan_context_get_offscreen_format         (GdkVulkanContext      *context,
-                                                                         GdkMemoryDepth         depth);
+void                    gdk_vulkan_context_set_draw_semaphore           (GdkVulkanContext      *context,
+                                                                         VkSemaphore            semaphore);
 
 #else /* !GDK_RENDERING_VULKAN */
 
@@ -120,4 +120,3 @@ gdk_display_ref_vulkan (GdkDisplay  *display,
 #endif /* !GDK_RENDERING_VULKAN */
 
 G_END_DECLS
-
