@@ -20,11 +20,13 @@
 #include "gdkpngprivate.h"
 
 #include <glib/gi18n-lib.h>
+#include "gdkcolorstateprivate.h"
 #include "gdkmemoryformatprivate.h"
-#include "gdkmemorytexture.h"
+#include "gdkmemorytexturebuilder.h"
 #include "gdkprofilerprivate.h"
 #include "gdktexturedownloaderprivate.h"
 #include "gsk/gl/fp16private.h"
+
 #include <png.h>
 #include <stdio.h>
 
@@ -126,6 +128,141 @@ png_simple_warning_callback (png_structp     png,
 {
 }
 
+/* }}} */
+/* {{{ Color profile handling */
+
+typedef struct
+{
+  gboolean cicp_chunk_read;
+  int color_primaries;
+  int transfer_function;
+  int matrix_coefficients;
+  int range;
+} CICPData;
+
+static int
+png_read_chunk_func (png_structp        png,
+                     png_unknown_chunkp chunk)
+{
+  if (strcmp ((char *) chunk->name, "cICP") == 0 &&
+      chunk->size == 4)
+    {
+      CICPData *cicp = png_get_user_chunk_ptr (png);
+
+      cicp->cicp_chunk_read = TRUE;
+
+      cicp->color_primaries = chunk->data[0];
+      cicp->transfer_function = chunk->data[1];
+      cicp->matrix_coefficients = chunk->data[2];
+      cicp->range = chunk->data[3];
+
+      return 1;
+    }
+
+  return 0;
+}
+
+static GdkColorState *
+gdk_png_get_color_state_from_cicp (const CICPData  *data,
+                                   GError         **error)
+{
+  GdkCicp cicp;
+
+  cicp.color_primaries = data->color_primaries;
+  cicp.transfer_function = data->transfer_function;
+  cicp.matrix_coefficients= data->matrix_coefficients;
+  cicp.range = data->range;
+
+  return gdk_color_state_new_for_cicp (&cicp, error);
+}
+
+static GdkColorState *
+gdk_png_get_color_state (png_struct  *png,
+                         png_info    *info,
+                         GError     **error)
+{
+  GdkColorState *color_state;
+  CICPData *cicp;
+  int intent;
+
+  cicp = png_get_user_chunk_ptr (png);
+
+  if (cicp->cicp_chunk_read)
+    {
+      GError *local_error = NULL;
+
+      color_state = gdk_png_get_color_state_from_cicp (cicp, &local_error);
+      if (color_state)
+        {
+          g_debug ("Color state from cICP data: %s", gdk_color_state_get_name (color_state));
+          return color_state;
+        }
+      else
+        {
+          g_set_error_literal (error,
+                               GDK_TEXTURE_ERROR, GDK_TEXTURE_ERROR_UNSUPPORTED_CONTENT,
+                               local_error->message);
+          g_error_free (local_error);
+        }
+    }
+
+#if 0
+  if (png_get_iCCP (png, info, &name, NULL, &icc_data, &icc_len))
+    {
+    }
+#endif
+
+  if (png_get_sRGB (png, info, &intent))
+    return gdk_color_state_ref (GDK_COLOR_STATE_SRGB);
+
+  /* If neither of those is valid, the result is sRGB */
+  if (!png_get_valid (png, info, PNG_INFO_gAMA) &&
+      !png_get_valid (png, info, PNG_INFO_cHRM))
+    return GDK_COLOR_STATE_SRGB;
+
+  g_debug ("Failed to find color state, assuming SRGB");
+
+  return GDK_COLOR_STATE_SRGB;
+}
+
+static void
+gdk_png_set_color_state (png_struct    *png,
+                         png_info      *info,
+                         GdkColorState *color_state,
+                         png_byte      *chunk_data)
+{
+  const GdkCicp *cicp;
+
+  cicp = gdk_color_state_get_cicp (color_state);
+
+  if (cicp)
+    {
+      png_unknown_chunk chunk = {
+        .name = { 'c', 'I', 'C', 'P', '\0' },
+        .data = chunk_data,
+        .size = 4,
+        .location = PNG_HAVE_IHDR,
+      };
+
+      chunk_data[0] = (png_byte) cicp->color_primaries;
+      chunk_data[1] = (png_byte) cicp->transfer_function;
+      chunk_data[2] = (png_byte) 0; /* png only supports this */
+      chunk_data[3] = (png_byte) cicp->range;
+
+      png_set_unknown_chunks (png, info, &chunk, 1);
+    }
+  else
+    {
+      /* unsupported color state. Fall back to sRGB */
+      gdk_color_state_unref (color_state);
+      color_state = gdk_color_state_ref (gdk_color_state_get_srgb ());
+    }
+
+  /* For good measure, we add an sRGB chunk too */
+  if (gdk_color_state_equal (color_state, GDK_COLOR_STATE_SRGB))
+    png_set_sRGB (png, info, PNG_sRGB_INTENT_PERCEPTUAL);
+}
+
 /* }}} */
 /* {{{ Public API */
 
@@ -143,12 +280,16 @@ gdk_load_png (GBytes      *bytes,
   gsize i, stride;
   int depth, color_type;
   int interlace;
+  GdkMemoryTextureBuilder *builder;
   GdkMemoryFormat format;
   guchar *buffer = NULL;
   guchar **row_pointers = NULL;
   GBytes *out_bytes;
+  GdkColorState *color_state;
   GdkTexture *texture;
   int bpp;
+  CICPData cicp = { FALSE, };
+
   G_GNUC_UNUSED gint64 before = GDK_PROFILER_CURRENT_TIME;
 
   io.data = (guchar *)g_bytes_get_data (bytes, &io.size);
@@ -169,6 +310,7 @@ gdk_load_png (GBytes      *bytes,
     g_error ("Out of memory");
 
   png_set_read_fn (png, &io, png_read_func);
+  png_set_read_user_chunk_fn (png, &cicp, png_read_chunk_func);
 
   if (sigsetjmp (png_jmpbuf (png), 1))
     {
@@ -266,6 +408,10 @@ gdk_load_png (GBytes      *bytes,
       return NULL;
     }
 
+  color_state = gdk_png_get_color_state (png, info, error);
+  if (color_state == NULL)
+    return NULL;
+
   bpp = gdk_memory_format_bytes_per_pixel (format);
   if (!g_size_checked_mul (&stride, width, bpp) ||
       !g_size_checked_add (&stride, stride, (8 - stride % 8) % 8))
@@ -281,6 +427,7 @@ gdk_load_png (GBytes      *bytes,
 
   if (!buffer || !row_pointers)
     {
+      gdk_color_state_unref (color_state);
       g_free (buffer);
       g_free (row_pointers);
       png_destroy_read_struct (&png, &info, NULL);
@@ -297,8 +444,17 @@ gdk_load_png (GBytes      *bytes,
   png_read_end (png, info);
 
   out_bytes = g_bytes_new_take (buffer, height * stride);
-  texture = gdk_memory_texture_new (width, height, format, out_bytes, stride);
+  builder = gdk_memory_texture_builder_new ();
+  gdk_memory_texture_builder_set_format (builder, format);
+  gdk_memory_texture_builder_set_color_state (builder, color_state);
+  gdk_memory_texture_builder_set_width (builder, width);
+  gdk_memory_texture_builder_set_height (builder, height);
+  gdk_memory_texture_builder_set_bytes (builder, out_bytes);
+  gdk_memory_texture_builder_set_stride (builder, stride);
+  texture = gdk_memory_texture_builder_build (builder);
+  g_object_unref (builder);
   g_bytes_unref (out_bytes);
+  gdk_color_state_unref (color_state);
 
   if (options && png_get_text (png, info, &text, &num_texts))
     {
@@ -337,11 +493,14 @@ gdk_save_png (GdkTexture *texture)
   GBytes *bytes;
   gsize stride;
   const guchar *data;
+  GdkColorState *color_state;
   int png_format;
   int depth;
+  png_byte chunk_data[4];
 
   width = gdk_texture_get_width (texture);
   height = gdk_texture_get_height (texture);
+  color_state = gdk_texture_get_color_state (texture);
   format = gdk_texture_get_format (texture);
 
   switch (format)
@@ -436,6 +595,8 @@ gdk_save_png (GdkTexture *texture)
   /* 2^31-1 is the maximum size for PNG files */
   png_set_user_limits (png, (1u << 31) - 1, (1u << 31) - 1);
 
+  png_set_keep_unknown_chunks (png, PNG_HANDLE_CHUNK_ALWAYS, NULL, 0);
+
   info = png_create_info_struct (png);
   if (!info)
     {
@@ -443,15 +604,13 @@ gdk_save_png (GdkTexture *texture)
       return NULL;
     }
 
-  gdk_texture_downloader_init (&downloader, texture);
-  gdk_texture_downloader_set_format (&downloader, format);
-  bytes = gdk_texture_downloader_download_bytes (&downloader, &stride);
-  gdk_texture_downloader_finish (&downloader);
-  data = g_bytes_get_data (bytes, NULL);
+  gdk_color_state_ref (color_state);
+  bytes = NULL;
 
   if (sigsetjmp (png_jmpbuf (png), 1))
     {
-      g_bytes_unref (bytes);
+      gdk_color_state_unref (color_state);
+      g_clear_pointer (&bytes, g_bytes_unref);
       g_free (io.data);
       png_destroy_read_struct (&png, &info, NULL);
       return NULL;
@@ -465,12 +624,20 @@ gdk_save_png (GdkTexture *texture)
                 PNG_COMPRESSION_TYPE_DEFAULT,
                 PNG_FILTER_TYPE_DEFAULT);
 
+  gdk_png_set_color_state (png, info, color_state, chunk_data);
+
   png_write_info (png, info);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
   png_set_swap (png);
 #endif
 
+  gdk_texture_downloader_init (&downloader, texture);
+  gdk_texture_downloader_set_format (&downloader, format);
+  bytes = gdk_texture_downloader_download_bytes (&downloader, &stride);
+  gdk_texture_downloader_finish (&downloader);
+  data = g_bytes_get_data (bytes, NULL);
+
   for (y = 0; y < height; y++)
     png_write_row (png, data + y * stride);
 
@@ -478,6 +645,7 @@ gdk_save_png (GdkTexture *texture)
 
   png_destroy_write_struct (&png, &info);
 
+  gdk_color_state_unref (color_state);
   g_bytes_unref (bytes);
 
   return g_bytes_new_take (io.data, io.size);
@@ -485,4 +653,4 @@ gdk_save_png (GdkTexture *texture)
 
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
