@@ -54,6 +54,7 @@
 #include "linux-dmabuf-unstable-v1-client-protocol.h"
 #include "presentation-time-client-protocol.h"
 
+#include "gsk/gskrectprivate.h"
 
 /**
  * GdkWaylandSurface:
@@ -657,6 +658,7 @@ static void
 gdk_wayland_surface_sync_opaque_region (GdkSurface *surface)
 {
   GdkWaylandSurface *impl = GDK_WAYLAND_SURFACE (surface);
+  GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (gdk_surface_get_display (surface));
   struct wl_region *wl_region = NULL;
 
   if (!impl->display_server.wl_surface)
@@ -679,16 +681,21 @@ gdk_wayland_surface_sync_opaque_region (GdkSurface *surface)
                 continue;
 
               if (sub->texture != NULL)
-                cairo_region_subtract_rectangle (region, &sub->dest);
+                {
+                  graphene_rect_t bounds;
+                  cairo_rectangle_int_t rect;
+
+                  gdk_subsurface_get_bounds (subsurface, &bounds);
+                  gsk_rect_to_cairo_grow (&bounds, &rect);
+                  cairo_region_subtract_rectangle (region, &rect);
+                }
             }
 
-          wl_region = wl_region_from_cairo_region (GDK_WAYLAND_DISPLAY (gdk_surface_get_display (surface)),
-                                                   region);
+          wl_region = wl_region_from_cairo_region (display, region);
           cairo_region_destroy (region);
         }
       else
-        wl_region = wl_region_from_cairo_region (GDK_WAYLAND_DISPLAY (gdk_surface_get_display (surface)),
-                                                 impl->opaque_region);
+        wl_region = wl_region_from_cairo_region (display, impl->opaque_region);
     }
 
   wl_surface_set_opaque_region (impl->display_server.wl_surface, wl_region);
@@ -765,6 +772,20 @@ gdk_wayland_surface_sync_viewport (GdkSurface *surface)
   self->viewport_dirty = FALSE;
 }
 
+static void
+gdk_wayland_surface_sync_color_state (GdkSurface *surface)
+{
+  GdkWaylandSurface *self = GDK_WAYLAND_SURFACE (surface);
+
+  if (!self->color_state_changed)
+    return;
+
+  gdk_wayland_color_surface_set_color_state (self->display_server.color,
+                                             gdk_surface_get_color_state (surface));
+
+  self->color_state_changed = FALSE;
+}
+
 void
 gdk_wayland_surface_sync (GdkSurface *surface)
 {
@@ -772,10 +793,11 @@ gdk_wayland_surface_sync (GdkSurface *surface)
   gdk_wayland_surface_sync_opaque_region (surface);
   gdk_wayland_surface_sync_input_region (surface);
   gdk_wayland_surface_sync_buffer_scale (surface);
+  gdk_wayland_surface_sync_color_state (surface);
   gdk_wayland_surface_sync_viewport (surface);
 }
 
-gboolean
+static gboolean
 gdk_wayland_surface_needs_commit (GdkSurface *surface)
 {
   GdkWaylandSurface *self = GDK_WAYLAND_SURFACE (surface);
@@ -784,9 +806,24 @@ gdk_wayland_surface_needs_commit (GdkSurface *surface)
          self->opaque_region_dirty ||
          self->input_region_dirty ||
          self->buffer_scale_dirty ||
+         self->color_state_changed ||
          self->viewport_dirty;
 }
 
+void
+gdk_wayland_surface_handle_empty_frame (GdkSurface *surface)
+{
+  if (!gdk_wayland_surface_needs_commit (surface))
+    return;
+
+  gdk_wayland_surface_sync (surface);
+  gdk_wayland_surface_request_frame (surface);
+
+  gdk_profiler_add_mark (GDK_PROFILER_CURRENT_TIME, 0, "Wayland surface commit", NULL);
+  gdk_wayland_surface_commit (surface);
+  gdk_wayland_surface_notify_committed (surface);
+}
+
 static void
 gdk_wayland_surface_fractional_scale_preferred_scale_cb (void *data,
                                                          struct wp_fractional_scale_v1 *fractional_scale,
@@ -799,6 +836,11 @@ gdk_wayland_surface_fractional_scale_preferred_scale_cb (void *data,
   gdk_wayland_surface_update_size (surface,
                                    surface->width, surface->height,
                                    &GDK_FRACTIONAL_SCALE_INIT (scale));
+
+  GDK_DISPLAY_DEBUG (gdk_surface_get_display (surface), EVENTS,
+                     "preferred fractional scale, surface %p scale %f",
+                     surface,
+                     gdk_fractional_scale_to_double (&GDK_FRACTIONAL_SCALE_INIT (scale)));
 }
 
 static const struct wp_fractional_scale_v1_listener fractional_scale_listener = {
@@ -815,8 +857,8 @@ surface_enter (void              *data,
   GdkDisplay *display = gdk_surface_get_display (surface);
   GdkMonitor *monitor;
 
-  GDK_DISPLAY_DEBUG(gdk_surface_get_display (surface), EVENTS,
-                    "surface enter, surface %p output %p", surface, output);
+  GDK_DISPLAY_DEBUG (gdk_surface_get_display (surface), EVENTS,
+                     "surface enter, surface %p output %p", surface, output);
 
   impl->display_server.outputs = g_slist_prepend (impl->display_server.outputs, output);
 
@@ -848,11 +890,53 @@ surface_leave (void              *data,
   gdk_surface_leave_monitor (surface, monitor);
 }
 
+static void
+surface_preferred_buffer_scale (void              *data,
+                                struct wl_surface *wl_surface,
+                                int32_t            factor)
+{
+  GdkSurface *surface = GDK_SURFACE (data);
+
+  GDK_DISPLAY_DEBUG (gdk_surface_get_display (surface), EVENTS,
+                     "preferred buffer scale, surface %p scale %d",
+                     surface, factor);
+}
+
+static void
+surface_preferred_buffer_transform (void              *data,
+                                    struct wl_surface *wl_surface,
+                                    uint32_t           transform)
+{
+  GdkSurface *surface = GDK_SURFACE (data);
+
+  GDK_DISPLAY_DEBUG (gdk_surface_get_display (surface), EVENTS,
+                     "preferred buffer transform, surface %p transform %s",
+                     surface, gdk_dihedral_get_name ((GdkDihedral) transform));
+}
+
 static const struct wl_surface_listener surface_listener = {
   surface_enter,
-  surface_leave
+  surface_leave,
+  surface_preferred_buffer_scale,
+  surface_preferred_buffer_transform,
 };
 
+static void
+preferred_changed (GdkWaylandColorSurface *color,
+                   GdkColorState          *color_state,
+                   gpointer                data)
+{
+  GdkWaylandSurface *self = GDK_WAYLAND_SURFACE (data);
+
+  GDK_DISPLAY_DEBUG (gdk_surface_get_display (GDK_SURFACE (self)), EVENTS,
+                     "preferred color state, surface %p color state %s",
+                     self, gdk_color_state_get_name (color_state));
+
+  gdk_surface_set_color_state (GDK_SURFACE (self), color_state);
+
+  self->color_state_changed = TRUE;
+}
+
 static void
 gdk_wayland_surface_create_wl_surface (GdkSurface *surface)
 {
@@ -877,6 +961,12 @@ gdk_wayland_surface_create_wl_surface (GdkSurface *surface)
           wp_viewporter_get_viewport (display_wayland->viewporter, wl_surface);
     }
 
+  if (display_wayland->color)
+    self->display_server.color = gdk_wayland_color_surface_new (display_wayland->color,
+                                                                wl_surface,
+                                                                preferred_changed,
+                                                                self);
+
   self->display_server.wl_surface = wl_surface;
 }
 
@@ -930,6 +1020,7 @@ gdk_wayland_surface_destroy_wl_surface (GdkWaylandSurface *self)
 
   g_clear_pointer (&self->display_server.viewport, wp_viewport_destroy);
   g_clear_pointer (&self->display_server.fractional_scale, wp_fractional_scale_v1_destroy);
+  g_clear_pointer (&self->display_server.color, gdk_wayland_color_surface_free);
 
   g_clear_pointer (&self->display_server.wl_surface, wl_surface_destroy);
 
@@ -1364,16 +1455,15 @@ void
 gdk_wayland_surface_ensure_wl_egl_window (GdkSurface *surface)
 {
   GdkWaylandSurface *impl = GDK_WAYLAND_SURFACE (surface);
+  int width, height;
 
-  if (impl->display_server.egl_window == NULL)
-    {
-      int width, height;
+  if (impl->display_server.egl_window != NULL)
+    return;
 
-      get_egl_window_size (surface, &width, &height);
-      impl->display_server.egl_window =
-        wl_egl_window_create (impl->display_server.wl_surface, width, height);
-      gdk_surface_set_egl_native_window (surface, impl->display_server.egl_window);
-    }
+  get_egl_window_size (surface, &width, &height);
+  impl->display_server.egl_window =
+    wl_egl_window_create (impl->display_server.wl_surface, width, height);
+  gdk_surface_set_egl_native_window (surface, impl->display_server.egl_window);
 }
 
 /* }}} */
@@ -1396,4 +1486,4 @@ gdk_wayland_surface_get_wl_surface (GdkSurface *surface)
 }
 
 /* }}}} */
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
