@@ -1087,25 +1087,25 @@ gsk_gl_command_queue_execute (GskGLCommandQueue    *self,
   glEnableVertexAttribArray (0);
   glVertexAttribPointer (0, 2, GL_FLOAT, GL_FALSE,
                          sizeof (GskGLDrawVertex),
-                         (void *) G_STRUCT_OFFSET (GskGLDrawVertex, position));
+                         GSIZE_TO_POINTER ((gsize) G_STRUCT_OFFSET (GskGLDrawVertex, position)));
 
   /* 1 = texture coord location */
   glEnableVertexAttribArray (1);
   glVertexAttribPointer (1, 2, GL_FLOAT, GL_FALSE,
                          sizeof (GskGLDrawVertex),
-                         (void *) G_STRUCT_OFFSET (GskGLDrawVertex, uv));
+                         GSIZE_TO_POINTER ((gsize) G_STRUCT_OFFSET (GskGLDrawVertex, uv)));
 
   /* 2 = color location */
   glEnableVertexAttribArray (2);
   glVertexAttribPointer (2, 4, GL_HALF_FLOAT, GL_FALSE,
                          sizeof (GskGLDrawVertex),
-                         (void *) G_STRUCT_OFFSET (GskGLDrawVertex, color));
+                         GSIZE_TO_POINTER ((gsize) G_STRUCT_OFFSET (GskGLDrawVertex, color)));
 
   /* 3 = color2 location */
   glEnableVertexAttribArray (3);
   glVertexAttribPointer (3, 4, GL_HALF_FLOAT, GL_FALSE,
                          sizeof (GskGLDrawVertex),
-                         (void *) G_STRUCT_OFFSET (GskGLDrawVertex, color2));
+                         GSIZE_TO_POINTER ((gsize) G_STRUCT_OFFSET (GskGLDrawVertex, color2)));
 
   /* Setup initial scissor clip */
   if (scissor != NULL && cairo_region_num_rectangles (scissor) > 0)
@@ -1483,6 +1483,7 @@ memory_format_gl_format (GskGLCommandQueue *self,
                          gboolean           ensure_mipmap,
                          gboolean          *out_can_mipmap,
                          GLint             *gl_internalformat,
+                         GLint             *gl_internalsrgbformat,
                          GLenum            *gl_format,
                          GLenum            *gl_type,
                          GLint              gl_swizzle[4])
@@ -1507,6 +1508,7 @@ memory_format_gl_format (GskGLCommandQueue *self,
       gdk_memory_format_gl_format (data_format,
                                    gdk_gl_context_get_use_es (self->context),
                                    gl_internalformat,
+                                   gl_internalsrgbformat,
                                    gl_format,
                                    gl_type,
                                    gl_swizzle);
@@ -1519,6 +1521,7 @@ memory_format_gl_format (GskGLCommandQueue *self,
                                         gdk_gl_context_get_use_es (self->context),
                                         &alt_format,
                                         gl_internalformat,
+                                        gl_internalsrgbformat,
                                         gl_format,
                                         gl_type,
                                         gl_swizzle))
@@ -1534,6 +1537,7 @@ memory_format_gl_format (GskGLCommandQueue *self,
           gdk_memory_format_gl_format (alt_format,
                                        gdk_gl_context_get_use_es (self->context),
                                        gl_internalformat,
+                                       gl_internalsrgbformat,
                                        gl_format,
                                        gl_type,
                                        gl_swizzle);
@@ -1552,6 +1556,7 @@ memory_format_gl_format (GskGLCommandQueue *self,
           gdk_memory_format_gl_format (fallbacks[i],
                                        gdk_gl_context_get_use_es (self->context),
                                        gl_internalformat,
+                                       gl_internalsrgbformat,
                                        gl_format,
                                        gl_type,
                                        gl_swizzle);
@@ -1624,7 +1629,7 @@ gsk_gl_command_queue_do_upload_texture_chunk (GskGLCommandQueue *self,
     {
       glTexSubImage2D (GL_TEXTURE_2D, 0, x, y, width, height, gl_format, gl_type, data);
     }
-  else if (stride % bpp == 0 && gdk_gl_context_has_feature (self->context, GDK_GL_FEATURE_UNPACK_SUBIMAGE))
+  else if (stride % bpp == 0)
     {
       glPixelStorei (GL_UNPACK_ROW_LENGTH, stride / bpp);
 
@@ -1670,7 +1675,7 @@ gsk_gl_command_queue_upload_texture_chunks (GskGLCommandQueue    *self,
   G_GNUC_UNUSED gint64 start_time = GDK_PROFILER_CURRENT_TIME;
   int width, height;
   GdkMemoryFormat data_format;
-  GLint gl_internalformat;
+  GLint gl_internalformat, gl_internalsrgbformat;
   GLenum gl_format;
   GLenum gl_type;
   GLint gl_swizzle[4];
@@ -1713,6 +1718,7 @@ gsk_gl_command_queue_upload_texture_chunks (GskGLCommandQueue    *self,
                                          ensure_mipmap,
                                          out_can_mipmap,
                                          &gl_internalformat,
+                                         &gl_internalsrgbformat,
                                          &gl_format,
                                          &gl_type,
                                          gl_swizzle);
