@@ -1,5 +1,4 @@
 /* GSK - The GTK Scene Kit
- *
  * Copyright 2016  Endless
  *
  * This library is free software; you can redistribute it and/or
@@ -52,26 +51,14 @@
 #include <graphene-gobject.h>
 #include <cairo-gobject.h>
 #include <gdk/gdk.h>
+#include "gdk/gdkdebugprivate.h"
 
-#ifdef GDK_WINDOWING_X11
-#include <gdk/x11/gdkx.h>
-#endif
 #ifdef GDK_WINDOWING_WAYLAND
 #include <gdk/wayland/gdkwayland.h>
 #endif
 #ifdef GDK_WINDOWING_BROADWAY
 #include "broadway/gskbroadwayrenderer.h"
 #endif
-#ifdef GDK_WINDOWING_MACOS
-#include <gdk/macos/gdkmacos.h>
-#endif
-#ifdef GDK_WINDOWING_WIN32
-#include <gdk/win32/gdkwin32.h>
-
-/* Remove these lines when OpenGL/ES 2.0 shader is ready */
-#include "win32/gdkprivate-win32.h"
-#include "win32/gdkdisplay-win32.h"
-#endif
 
 typedef struct
 {
@@ -189,7 +176,7 @@ gsk_renderer_class_init (GskRendererClass *klass)
   gobject_class->dispose = gsk_renderer_dispose;
 
   /**
-   * GskRenderer:realized: (attributes org.gtk.Property.get=gsk_renderer_is_realized)
+   * GskRenderer:realized: (getter is_realized)
    *
    * Whether the renderer has been associated with a surface or draw context.
    */
@@ -199,7 +186,7 @@ gsk_renderer_class_init (GskRendererClass *klass)
                           G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GskRenderer:surface: (attributes org.gtk.Property.get=gsk_renderer_get_surface)
+   * GskRenderer:surface:
    *
    * The surface associated with renderer.
    */
@@ -221,7 +208,7 @@ gsk_renderer_init (GskRenderer *self)
 }
 
 /**
- * gsk_renderer_get_surface: (attributes org.gtk.Method.get_property=surface)
+ * gsk_renderer_get_surface:
  * @renderer: a `GskRenderer`
  *
  * Retrieves the `GdkSurface` set using gsk_enderer_realize().
@@ -241,7 +228,7 @@ gsk_renderer_get_surface (GskRenderer *renderer)
 }
 
 /**
- * gsk_renderer_is_realized: (attributes org.gtk.Method.get_property=realized)
+ * gsk_renderer_is_realized: (get-property realized)
  * @renderer: a `GskRenderer`
  *
  * Checks whether the @renderer is realized or not.
@@ -429,21 +416,6 @@ gsk_renderer_render_texture (GskRenderer           *renderer,
 
   texture = GSK_RENDERER_GET_CLASS (renderer)->render_texture (renderer, root, viewport);
 
-  if (GSK_RENDERER_DEBUG_CHECK (renderer, RENDERER))
-    {
-      GString *buf = g_string_new ("*** Texture stats ***\n\n");
-
-      gsk_profiler_append_counters (priv->profiler, buf);
-      g_string_append_c (buf, '\n');
-
-      gsk_profiler_append_timers (priv->profiler, buf);
-      g_string_append_c (buf, '\n');
-
-      g_print ("%s\n***\n\n", buf->str);
-
-      g_string_free (buf, TRUE);
-    }
-
   return texture;
 }
 
@@ -488,8 +460,7 @@ gsk_renderer_render (GskRenderer          *renderer,
 
   clip = cairo_region_copy (region);
 
-  if (renderer_class->supports_offload &&
-      !GSK_RENDERER_DEBUG_CHECK (renderer, OFFLOAD_DISABLE))
+  if (renderer_class->supports_offload && gdk_has_feature (GDK_FEATURE_OFFLOAD))
     offload = gsk_offload_new (priv->surface, root, clip);
   else
     offload = NULL;
@@ -510,21 +481,6 @@ gsk_renderer_render (GskRenderer          *renderer,
 
   renderer_class->render (renderer, root, clip);
 
-  if (GSK_RENDERER_DEBUG_CHECK (renderer, RENDERER))
-    {
-      GString *buf = g_string_new ("*** Frame stats ***\n\n");
-
-      gsk_profiler_append_counters (priv->profiler, buf);
-      g_string_append_c (buf, '\n');
-
-      gsk_profiler_append_timers (priv->profiler, buf);
-      g_string_append_c (buf, '\n');
-
-      g_print ("%s\n***\n\n", buf->str);
-
-      g_string_free (buf, TRUE);
-    }
-
   g_clear_pointer (&priv->prev_node, gsk_render_node_unref);
   cairo_region_destroy (clip);
   g_clear_pointer (&offload, gsk_offload_free);
@@ -577,16 +533,14 @@ get_renderer_for_name (const char *renderer_name)
 #else
       g_print ("  broadway - Disabled during GTK build\n");
 #endif
-      g_print ("   cairo - Use the Cairo fallback renderer\n");
-      g_print ("  opengl - Use the OpenGL renderer\n");
-      g_print ("      gl - Use the OpenGL renderer\n");
-      g_print ("     ngl - Use the new OpenGL renderer\n");
+      g_print ("     cairo - Use the Cairo fallback renderer\n");
+      g_print ("       ngl - Use the OpenGL renderer\n");
 #ifdef GDK_RENDERING_VULKAN
-      g_print ("  vulkan - Use the Vulkan renderer\n");
+      g_print ("    vulkan - Use the Vulkan renderer\n");
 #else
-      g_print ("  vulkan - Disabled during GTK build\n");
+      g_print ("    vulkan - Disabled during GTK build\n");
 #endif
-      g_print ("    help - Print this help\n\n");
+      g_print ("      help - Print this help\n\n");
       g_print ("Other arguments will cause a warning and be ignored.\n");
     }
   else
@@ -610,12 +564,17 @@ get_renderer_for_display (GdkSurface *surface)
 static GType
 get_renderer_for_env_var (GdkSurface *surface)
 {
-  static GType env_var_type = G_TYPE_NONE;
+  static GType env_var_type = G_TYPE_INVALID;
 
-  if (env_var_type == G_TYPE_NONE)
+  if (env_var_type == G_TYPE_INVALID)
     {
       const char *renderer_name = g_getenv ("GSK_RENDERER");
       env_var_type = get_renderer_for_name (renderer_name);
+      if (env_var_type != G_TYPE_INVALID)
+        GSK_DEBUG (RENDERER,
+                   "Environment variable GSK_RENDERER=%s set, trying %s",
+                   renderer_name,
+                   g_type_name (env_var_type));
     }
 
   return env_var_type;
@@ -633,24 +592,52 @@ get_renderer_for_backend (GdkSurface *surface)
 }
 
 static gboolean
-gl_software_rendering (GdkSurface *surface)
+gl_supported_platform (GdkSurface *surface,
+                       GType       renderer_type,
+                       gboolean    as_fallback)
 {
   GdkDisplay *display = gdk_surface_get_display (surface);
   GdkGLContext *context;
+  GError *error = NULL;
 
-  if (!gdk_display_prepare_gl (display, NULL))
-    return G_TYPE_INVALID;
+  if (!gdk_display_prepare_gl (display, &error))
+    {
+      GSK_DEBUG (RENDERER, "Not using GL%s: %s",
+                 as_fallback ? " as fallback" : "",
+                 error->message);
+      g_clear_error (&error);
+      return FALSE;
+    }
+
+  if (as_fallback)
+    return TRUE;
 
   context = gdk_display_get_gl_context (display);
   gdk_gl_context_make_current (context);
 
-  return strstr ((const char *) glGetString (GL_RENDERER), "llvmpipe") != NULL;
+  if (strstr ((const char *) glGetString (GL_RENDERER), "llvmpipe") != NULL)
+    {
+      GSK_DEBUG (RENDERER, "Not using '%s': renderer is llvmpipe",
+                 g_type_name (renderer_type));
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 static GType
 get_renderer_for_gl (GdkSurface *surface)
 {
-  if (gl_software_rendering (surface))
+  if (!gl_supported_platform (surface, gsk_ngl_renderer_get_type (), FALSE))
+    return G_TYPE_INVALID;
+
+  return gsk_ngl_renderer_get_type ();
+}
+
+static GType
+get_renderer_for_gl_fallback (GdkSurface *surface)
+{
+  if (!gl_supported_platform (surface, gsk_ngl_renderer_get_type (), TRUE))
     return G_TYPE_INVALID;
 
   return gsk_ngl_renderer_get_type ();
@@ -658,38 +645,76 @@ get_renderer_for_gl (GdkSurface *surface)
 
 #ifdef GDK_RENDERING_VULKAN
 static gboolean
-vulkan_software_rendering (GdkSurface *surface)
+vulkan_supported_platform (GdkSurface *surface,
+                           GType       renderer_type,
+                           gboolean    as_fallback)
 {
   GdkDisplay *display = gdk_surface_get_display (surface);
   VkPhysicalDeviceProperties props;
+  GError *error = NULL;
 
-  if (!gdk_display_init_vulkan (display, NULL))
-    return G_TYPE_INVALID;
+#ifdef GDK_WINDOWING_WAYLAND
+  if (!GDK_IS_WAYLAND_DISPLAY (gdk_surface_get_display (surface)) && !as_fallback)
+    {
+      GSK_DEBUG (RENDERER, "Not using '%s': platform is not Wayland",
+                 g_type_name (renderer_type));
+      return FALSE;
+    }
+#endif
+
+  if (!gdk_display_prepare_vulkan (display, &error))
+    {
+      GSK_DEBUG (RENDERER, "Not using Vulkan%s: %s",
+                 as_fallback ? " as fallback" : "",
+                 error->message);
+      g_clear_error (&error);
+      return FALSE;
+    }
+
+  if (as_fallback)
+    return TRUE;
 
   vkGetPhysicalDeviceProperties (display->vk_physical_device, &props);
 
-  return props.deviceType == VK_PHYSICAL_DEVICE_TYPE_CPU;
-}
+  if (props.deviceType == VK_PHYSICAL_DEVICE_TYPE_CPU)
+    {
+      GSK_DEBUG (RENDERER, "Not using '%s': device is CPU",
+                 g_type_name (renderer_type));
+      return FALSE;
+    }
+
+#ifdef HAVE_DMABUF
+  gdk_vulkan_init_dmabuf (display);
+  if (!display->vk_dmabuf_formats ||
+      gdk_dmabuf_formats_get_n_formats (display->vk_dmabuf_formats) == 0)
+    {
+      GSK_DEBUG (RENDERER, "Not using '%s': no dmabuf support",
+                 g_type_name (renderer_type));
+      return FALSE;
+    }
 #endif
 
+  return TRUE;
+}
+
 static GType
 get_renderer_for_vulkan (GdkSurface *surface)
 {
-#ifdef GDK_RENDERING_VULKAN
-  if (vulkan_software_rendering (surface))
+  if (!vulkan_supported_platform (surface, GSK_TYPE_VULKAN_RENDERER, FALSE))
     return G_TYPE_INVALID;
 
   return GSK_TYPE_VULKAN_RENDERER;
-#else
-  return G_TYPE_INVALID;
-#endif
 }
 
 static GType
-get_renderer_for_gles2 (GdkSurface *surface)
+get_renderer_for_vulkan_fallback (GdkSurface *surface)
 {
-  return GSK_TYPE_GL_RENDERER;
+  if (!vulkan_supported_platform (surface, GSK_TYPE_VULKAN_RENDERER, TRUE))
+    return G_TYPE_INVALID;
+
+  return GSK_TYPE_VULKAN_RENDERER;
 }
+#endif
 
 static GType
 get_renderer_fallback (GdkSurface *surface)
@@ -703,9 +728,14 @@ static struct {
   { get_renderer_for_display },
   { get_renderer_for_env_var },
   { get_renderer_for_backend },
-  { get_renderer_for_gl },
-  { get_renderer_for_gles2 },
+#ifdef GDK_RENDERING_VULKAN
   { get_renderer_for_vulkan },
+#endif
+  { get_renderer_for_gl },
+#ifdef GDK_RENDERING_VULKAN
+  { get_renderer_for_vulkan_fallback },
+#endif
+  { get_renderer_for_gl_fallback },
   { get_renderer_fallback },
 };
 
@@ -743,17 +773,19 @@ gsk_renderer_new_for_surface (GdkSurface *surface)
 
       if (gsk_renderer_realize (renderer, surface, &error))
         {
-          GSK_RENDERER_DEBUG (renderer, RENDERER,
-                              "Using renderer of type '%s' for surface '%s'",
-                              G_OBJECT_TYPE_NAME (renderer),
-                              G_OBJECT_TYPE_NAME (surface));
+          GSK_DEBUG (RENDERER,
+                     "Using renderer '%s' for surface '%s'",
+                     G_OBJECT_TYPE_NAME (renderer),
+                     G_OBJECT_TYPE_NAME (surface));
           return renderer;
         }
 
-      g_message ("Failed to realize renderer of type '%s' for surface '%s': %s\n",
+      GSK_DEBUG (RENDERER,
+                 "Failed to realize renderer '%s' for surface '%s': %s",
                  G_OBJECT_TYPE_NAME (renderer),
                  G_OBJECT_TYPE_NAME (surface),
                  error->message);
+
       g_object_unref (renderer);
       g_clear_error (&error);
     }
