@@ -146,11 +146,8 @@ collect_states (GtkAtSpiContext    *self,
   set_atspi_state (&states, ATSPI_STATE_VISIBLE);
   set_atspi_state (&states, ATSPI_STATE_SHOWING);
 
-  if (ctx->accessible_role == GTK_ACCESSIBLE_ROLE_APPLICATION)
-    {
-      if (gtk_accessible_get_platform_state (accessible, GTK_ACCESSIBLE_PLATFORM_STATE_ACTIVE))
-        set_atspi_state (&states, ATSPI_STATE_ACTIVE);
-    }
+  if (gtk_accessible_get_platform_state (accessible, GTK_ACCESSIBLE_PLATFORM_STATE_ACTIVE))
+    set_atspi_state (&states, ATSPI_STATE_ACTIVE);
 
   if (ctx->accessible_role == GTK_ACCESSIBLE_ROLE_TEXT_BOX ||
       ctx->accessible_role == GTK_ACCESSIBLE_ROLE_SEARCH_BOX ||
@@ -530,6 +527,15 @@ handle_accessible_method (GDBusConnection       *connection,
       g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{ss}"));
       g_variant_builder_add (&builder, "{ss}", "toolkit", "GTK");
 
+      if (gtk_at_context_has_accessible_property (GTK_AT_CONTEXT (self), GTK_ACCESSIBLE_PROPERTY_LEVEL))
+        {
+          GtkAccessibleValue *value = gtk_at_context_get_accessible_property (GTK_AT_CONTEXT (self),
+                                                                              GTK_ACCESSIBLE_PROPERTY_LEVEL);
+          char *level = g_strdup_printf ("%d", gtk_int_accessible_value_get (value));
+          g_variant_builder_add (&builder, "{ss}", "level", level);
+          g_free (level);
+        }
+
       if (gtk_at_context_has_accessible_property (GTK_AT_CONTEXT (self), GTK_ACCESSIBLE_PROPERTY_PLACEHOLDER))
         {
           GtkAccessibleValue *value;
@@ -720,6 +726,16 @@ handle_accessible_get_property (GDBusConnection       *connection,
     res = get_parent_context_ref (accessible);
   else if (g_strcmp0 (property_name, "ChildCount") == 0)
     res = g_variant_new_int32 (gtk_at_spi_context_get_child_count (self));
+  else if (g_strcmp0 (property_name, "HelpText") == 0)
+    {
+      if (gtk_at_context_has_accessible_property (GTK_AT_CONTEXT (self), GTK_ACCESSIBLE_PROPERTY_HELP_TEXT))
+        {
+          GtkAccessibleValue *value = gtk_at_context_get_accessible_property (GTK_AT_CONTEXT (self), GTK_ACCESSIBLE_PROPERTY_HELP_TEXT);
+          res = g_variant_new_string (gtk_string_accessible_value_get (value));
+        }
+      else
+        res = g_variant_new_string ("");
+    }
   else
     g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
                  "Unknown property '%s'", property_name);
@@ -741,7 +757,7 @@ emit_text_changed (GtkAtSpiContext *self,
                    int              end,
                    const char      *text)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -761,7 +777,7 @@ emit_text_selection_changed (GtkAtSpiContext *self,
                              const char      *kind,
                              int              cursor_position)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   if (strcmp (kind, "text-caret-moved") == 0)
@@ -788,7 +804,7 @@ static void
 emit_selection_changed (GtkAtSpiContext *self,
                         const char      *kind)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -806,7 +822,7 @@ emit_state_changed (GtkAtSpiContext *self,
                     const char      *name,
                     gboolean         enabled)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -822,7 +838,7 @@ emit_state_changed (GtkAtSpiContext *self,
 static void
 emit_defunct (GtkAtSpiContext *self)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -841,20 +857,16 @@ emit_property_changed (GtkAtSpiContext *self,
 {
   GVariant *value_owned = g_variant_ref_sink (value);
 
-  if (self->connection == NULL)
-    {
-      g_variant_unref (value_owned);
-      return;
-    }
+  if (self->connection != NULL && gtk_at_spi_root_has_event_listeners (self->root))
+    g_dbus_connection_emit_signal (self->connection,
+                                   NULL,
+                                   self->context_path,
+                                   "org.a11y.atspi.Event.Object",
+                                   "PropertyChange",
+                                   g_variant_new ("(siiva{sv})",
+                                                  name, 0, 0, value_owned, NULL),
+                                   NULL);
 
-  g_dbus_connection_emit_signal (self->connection,
-                                 NULL,
-                                 self->context_path,
-                                 "org.a11y.atspi.Event.Object",
-                                 "PropertyChange",
-                                 g_variant_new ("(siiva{sv})",
-                                                name, 0, 0, value_owned, NULL),
-                                 NULL);
   g_variant_unref (value_owned);
 }
 
@@ -865,7 +877,7 @@ emit_bounds_changed (GtkAtSpiContext *self,
                      int              width,
                      int              height)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -885,25 +897,25 @@ emit_children_changed (GtkAtSpiContext         *self,
                        GtkAccessibleChildState  state)
 {
   /* If we don't have a connection on either contexts, we cannot emit a signal */
-  if (self->connection == NULL || child_context->connection == NULL)
+  if (self->connection == NULL ||
+      child_context->connection == NULL ||
+      !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
-  GVariant *context_ref = gtk_at_spi_context_to_ref (self);
   GVariant *child_ref = gtk_at_spi_context_to_ref (child_context);
 
   gtk_at_spi_emit_children_changed (self->connection,
                                     self->context_path,
                                     state,
                                     idx,
-                                    child_ref,
-                                    context_ref);
+                                    child_ref);
 }
 
 static void
 emit_window_event (GtkAtSpiContext *self,
                    const char      *event_type)
 {
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -1136,7 +1148,7 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
     }
 
   if (changed_properties & GTK_ACCESSIBLE_PROPERTY_CHANGE_DESCRIPTION)
-  {
+    {
       char *label = gtk_at_context_get_description (GTK_AT_CONTEXT (self));
       GVariant *v = g_variant_new_take_string (label);
       emit_property_changed (self, "accessible-description", v);
@@ -1149,6 +1161,14 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
                              "accessible-value",
                              g_variant_new_double (gtk_number_accessible_value_get (value)));
     }
+
+  if (changed_properties & GTK_ACCESSIBLE_PROPERTY_CHANGE_HELP_TEXT)
+    {
+      value = gtk_accessible_attribute_set_get_value (properties, GTK_ACCESSIBLE_PROPERTY_HELP_TEXT);
+      emit_property_changed (self,
+                             "accessible-help-text",
+                             g_variant_new_string (gtk_string_accessible_value_get (value)));
+    }
 }
 
 static void
@@ -1560,7 +1580,7 @@ gtk_at_spi_context_update_caret_position (GtkATContext *context)
   GtkAccessibleText *accessible_text = GTK_ACCESSIBLE_TEXT (accessible);
   guint offset;
 
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   offset = gtk_accessible_text_get_caret_position (accessible_text);
@@ -1584,7 +1604,7 @@ gtk_at_spi_context_update_selection_bound (GtkATContext *context)
 {
   GtkAtSpiContext *self = GTK_AT_SPI_CONTEXT (context);
 
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   g_dbus_connection_emit_signal (self->connection,
@@ -1609,7 +1629,7 @@ gtk_at_spi_context_update_text_contents (GtkATContext *context,
 {
   GtkAtSpiContext *self = GTK_AT_SPI_CONTEXT (context);
 
-  if (self->connection == NULL)
+  if (self->connection == NULL || !gtk_at_spi_root_has_event_listeners (self->root))
     return;
 
   GtkAccessible *accessible = gtk_at_context_get_accessible (context);
@@ -1993,4 +2013,4 @@ gtk_at_spi_context_get_child_count (GtkAtSpiContext *self)
 }
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
