@@ -29,21 +29,22 @@
 #include "gtkshortcutswindowprivate.h"
 #include "gtktooltipprivate.h"
 
+#include <glib/gi18n-lib.h>
+
 /**
  * GtkApplicationWindow:
  *
- * `GtkApplicationWindow` is a `GtkWindow` subclass that integrates with
- * `GtkApplication`.
+ * A `GtkWindow` subclass that integrates with `GtkApplication`.
  *
  * Notably, `GtkApplicationWindow` can handle an application menubar.
  *
- * This class implements the `GActionGroup` and `GActionMap` interfaces,
- * to let you add window-specific actions that will be exported by the
- * associated [class@Gtk.Application], together with its application-wide
+ * This class implements the [iface@Gio.ActionGroup] and [iface@Gio.ActionMap]
+ * interfaces, to let you add window-specific actions that will be exported
+ * by the associated [class@Gtk.Application], together with its application-wide
  * actions. Window-specific actions are prefixed with the “win.”
  * prefix and application-wide actions are prefixed with the “app.”
  * prefix. Actions must be addressed with the prefixed name when
- * referring to them from a `GMenuModel`.
+ * referring to them from a menu model.
  *
  * Note that widgets that are placed inside a `GtkApplicationWindow`
  * can also activate these actions, if they implement the
@@ -230,6 +231,9 @@ gtk_application_window_update_menubar (GtkApplicationWindow *window)
       g_menu_append_section (combined, NULL, G_MENU_MODEL (priv->menubar_section));
 
       priv->menubar = gtk_popover_menu_bar_new_from_model (G_MENU_MODEL (combined));
+      gtk_accessible_update_property (GTK_ACCESSIBLE (priv->menubar),
+                                      GTK_ACCESSIBLE_PROPERTY_LABEL, _("Menu bar"),
+                                      -1);
       gtk_widget_set_parent (priv->menubar, GTK_WIDGET (window));
       g_object_unref (combined);
     }
@@ -668,14 +672,14 @@ gtk_application_window_class_init (GtkApplicationWindowClass *class)
   object_class->dispose = gtk_application_window_dispose;
 
   /**
-   * GtkApplicationWindow:show-menubar: (attributes org.gtk.Property.get=gtk_application_window_get_show_menubar org.gtk.Property.set=gtk_application_window_set_show_menubar)
+   * GtkApplicationWindow:show-menubar:
    *
-   * If this property is %TRUE, the window will display a menubar
+   * If this property is true, the window will display a menubar
    * unless it is shown by the desktop shell.
    *
    * See [method@Gtk.Application.set_menubar].
    *
-   * If %FALSE, the window will not display a menubar, regardless
+   * If false, the window will not display a menubar, regardless
    * of whether the desktop shell is showing it or not.
    */
   gtk_application_window_properties[PROP_SHOW_MENUBAR] =
@@ -686,7 +690,7 @@ gtk_application_window_class_init (GtkApplicationWindowClass *class)
 
 /**
  * gtk_application_window_new:
- * @application: a `GtkApplication`
+ * @application: an application
  *
  * Creates a new `GtkApplicationWindow`.
  *
@@ -703,13 +707,13 @@ gtk_application_window_new (GtkApplication *application)
 }
 
 /**
- * gtk_application_window_get_show_menubar: (attributes org.gtk.Method.get_property=show-menubar)
- * @window: a `GtkApplicationWindow`
+ * gtk_application_window_get_show_menubar:
+ * @window: an application window
  *
  * Returns whether the window will display a menubar for the app menu
  * and menubar as needed.
  *
- * Returns: %TRUE if @window will display a menubar when needed
+ * Returns: True if the window will display a menubar when needed
  */
 gboolean
 gtk_application_window_get_show_menubar (GtkApplicationWindow *window)
@@ -719,8 +723,8 @@ gtk_application_window_get_show_menubar (GtkApplicationWindow *window)
 }
 
 /**
- * gtk_application_window_set_show_menubar: (attributes org.gtk.Method.set_property=show-menubar)
- * @window: a `GtkApplicationWindow`
+ * gtk_application_window_set_show_menubar:
+ * @window: an application window
  * @show_menubar: whether to show a menubar when needed
  *
  * Sets whether the window will display a menubar for the app menu
@@ -747,14 +751,14 @@ gtk_application_window_set_show_menubar (GtkApplicationWindow *window,
 
 /**
  * gtk_application_window_get_id:
- * @window: a `GtkApplicationWindow`
+ * @window: an application window
  *
  * Returns the unique ID of the window.
  *
  *  If the window has not yet been added to a `GtkApplication`, returns `0`.
  *
- * Returns: the unique ID for @window, or `0` if the window
- *   has not yet been added to a `GtkApplication`
+ * Returns: the unique ID for the window, or `0` if the window
+ *   has not yet been added to an application
  */
 guint
 gtk_application_window_get_id (GtkApplicationWindow *window)
@@ -788,15 +792,15 @@ show_help_overlay (GSimpleAction *action,
 
 /**
  * gtk_application_window_set_help_overlay:
- * @window: a `GtkApplicationWindow`
- * @help_overlay: (nullable): a `GtkShortcutsWindow`
+ * @window: an application window
+ * @help_overlay: (nullable): a shortcuts window
  *
  * Associates a shortcuts window with the application window.
  *
  * Additionally, sets up an action with the name
  * `win.show-help-overlay` to present it.
  *
- * @window takes responsibility for destroying @help_overlay.
+ * The window takes responsibility for destroying the help overlay.
  */
 void
 gtk_application_window_set_help_overlay (GtkApplicationWindow *window,
@@ -832,14 +836,14 @@ gtk_application_window_set_help_overlay (GtkApplicationWindow *window,
 
 /**
  * gtk_application_window_get_help_overlay:
- * @window: a `GtkApplicationWindow`
+ * @window: an application window
  *
  * Gets the `GtkShortcutsWindow` that is associated with @window.
  *
  * See [method@Gtk.ApplicationWindow.set_help_overlay].
  *
  * Returns: (transfer none) (nullable): the help overlay associated
- *   with @window
+ *   with the window
  */
 GtkShortcutsWindow *
 gtk_application_window_get_help_overlay (GtkApplicationWindow *window)
