@@ -92,7 +92,7 @@ gtk_bitset_unref (GtkBitset *self)
     return;
 
   ra_clear (&self->roaring.high_low_container);
-  g_slice_free (GtkBitset, self);
+  g_free (self);
 }
 
 /**
@@ -271,7 +271,7 @@ gtk_bitset_new_empty (void)
 {
   GtkBitset *self;
 
-  self = g_slice_new0 (GtkBitset);
+  self = g_new0 (GtkBitset, 1);
 
   self->ref_count = 1;
 
@@ -319,7 +319,9 @@ gtk_bitset_copy (const GtkBitset *self)
   g_return_val_if_fail (self != NULL, NULL);
 
   copy = gtk_bitset_new_empty ();
-  roaring_bitmap_overwrite (&copy->roaring, &self->roaring);
+
+  if (!gtk_bitset_is_empty (self))
+    roaring_bitmap_overwrite (&copy->roaring, &self->roaring);
 
   return copy;
 }
