@@ -19,7 +19,7 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 #include "config.h"
@@ -207,7 +207,7 @@ gtk_drag_source_set_property (GObject      *object,
                               GParamSpec   *pspec)
 {
   GtkDragSource *source = GTK_DRAG_SOURCE (object);
-  
+
   switch (prop_id)
     {
     case PROP_CONTENT:
@@ -333,7 +333,7 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
   class->prepare = gtk_drag_source_prepare;
 
   /**
-   * GtkDragSource:content: (attributes org.gtk.Property.get=gtk_drag_source_get_content org.gtk.Property.set=gtk_drag_source_set_content)
+   * GtkDragSource:content:
    *
    * The data that is offered by drag operations from this source.
    */
@@ -343,7 +343,7 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkDragSource:actions: (attributes org.gtk.Property.get=gtk_drag_source_get_actions org.gtk.Property.set=gtk_drag_source_set_actions)
+   * GtkDragSource:actions:
    *
    * The actions that are supported by drag operations from the source.
    *
@@ -378,9 +378,12 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET (GtkDragSourceClass, prepare),
                     g_signal_accumulator_first_wins, NULL,
-                    NULL,
+                    _gtk_marshal_OBJECT__DOUBLE_DOUBLE,
                     GDK_TYPE_CONTENT_PROVIDER, 2,
                     G_TYPE_DOUBLE, G_TYPE_DOUBLE);
+  g_signal_set_va_marshaller (signals[PREPARE],
+                              GTK_TYPE_DRAG_SOURCE,
+                              _gtk_marshal_OBJECT__DOUBLE_DOUBLEv);
 
   /**
    * GtkDragSource::drag-begin:
@@ -421,10 +424,13 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
                     G_SIGNAL_RUN_LAST,
                     0,
                     NULL, NULL,
-                    NULL,
+                    _gtk_marshal_VOID__OBJECT_BOOLEAN,
                     G_TYPE_NONE, 2,
                     GDK_TYPE_DRAG,
                     G_TYPE_BOOLEAN);
+  g_signal_set_va_marshaller (signals[DRAG_END],
+                              GTK_TYPE_DRAG_SOURCE,
+                              _gtk_marshal_VOID__OBJECT_BOOLEANv);
 
   /**
    * GtkDragSource::drag-cancel:
@@ -450,6 +456,9 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
                     G_TYPE_BOOLEAN, 2,
                     GDK_TYPE_DRAG,
                     GDK_TYPE_DRAG_CANCEL_REASON);
+  g_signal_set_va_marshaller (signals[DRAG_CANCEL],
+                              G_TYPE_FROM_CLASS (class),
+                              _gtk_marshal_BOOLEAN__OBJECT_ENUMv);
 }
 
 static GdkContentProvider *
@@ -564,7 +573,7 @@ gtk_drag_source_drag_begin (GtkDragSource *source)
   GtkWidget *widget;
   GdkDevice *device, *pointer;
   GdkSeat *seat;
-  double x, y;
+  graphene_point_t p;
   GtkNative *native;
   GdkSurface *surface;
   double px, py;
@@ -583,11 +592,15 @@ gtk_drag_source_drag_begin (GtkDragSource *source)
   native = gtk_widget_get_native (widget);
   surface = gtk_native_get_surface (native);
 
-  gtk_widget_translate_coordinates (widget, GTK_WIDGET (native), source->start_x, source->start_y, &x, &y);
+  if (!gtk_widget_compute_point (widget, GTK_WIDGET (native),
+                                 &GRAPHENE_POINT_INIT (source->start_x, source->start_y),
+                                 &p))
+    return;
+
   gdk_surface_get_device_position (surface, pointer, &px, &py, NULL);
 
-  dx = round (px - x);
-  dy = round (py - y);
+  dx = round (px - p.x);
+  dy = round (py - p.y);
 
   g_signal_emit (source, signals[PREPARE], 0, source->start_x, source->start_y, &content);
   if (!content)
@@ -629,7 +642,7 @@ gtk_drag_source_new (void)
 }
 
 /**
- * gtk_drag_source_get_content: (attributes org.gtk.Method.get_property=content)
+ * gtk_drag_source_get_content:
  * @source: a `GtkDragSource`
  *
  * Gets the current content provider of a `GtkDragSource`.
@@ -645,7 +658,7 @@ gtk_drag_source_get_content (GtkDragSource *source)
 }
 
 /**
- * gtk_drag_source_set_content: (attributes org.gtk.Method.set_property=content)
+ * gtk_drag_source_set_content:
  * @source: a `GtkDragSource`
  * @content: (nullable): a `GdkContentProvider`
  *
@@ -673,7 +686,7 @@ gtk_drag_source_set_content (GtkDragSource      *source,
 }
 
 /**
- * gtk_drag_source_get_actions: (attributes org.gtk.Method.get_property=actions)
+ * gtk_drag_source_get_actions:
  * @source: a `GtkDragSource`
  *
  * Gets the actions that are currently set on the `GtkDragSource`.
@@ -689,7 +702,7 @@ gtk_drag_source_get_actions (GtkDragSource *source)
 }
 
 /**
- * gtk_drag_source_set_actions: (attributes org.gtk.Method.set_property=actions)
+ * gtk_drag_source_set_actions:
  * @source: a `GtkDragSource`
  * @actions: the actions to offer
  *
@@ -788,7 +801,7 @@ gtk_drag_source_drag_cancel (GtkDragSource *source)
 
 /**
  * gtk_drag_check_threshold: (method)
- * @widget: a `GtkWidget`
+ * @widget: a widget
  * @start_x: X coordinate of start of drag
  * @start_y: Y coordinate of start of drag
  * @current_x: current X coordinate
@@ -796,7 +809,7 @@ gtk_drag_source_drag_cancel (GtkDragSource *source)
  *
  * Checks to see if a drag movement has passed the GTK drag threshold.
  *
- * Returns: %TRUE if the drag threshold has been passed.
+ * Returns: true if the drag threshold has been passed
  */
 gboolean
 gtk_drag_check_threshold (GtkWidget *widget,
