@@ -215,7 +215,7 @@ G_DEFINE_TYPE_WITH_CODE (GtkFileChooserNative, gtk_file_chooser_native, GTK_TYPE
 
 
 /**
- * gtk_file_chooser_native_get_accept_label: (attributes org.gtk.Method.get_property=accept-label)
+ * gtk_file_chooser_native_get_accept_label:
  * @self: a `GtkFileChooserNative`
  *
  * Retrieves the custom label text for the accept button.
@@ -233,7 +233,7 @@ gtk_file_chooser_native_get_accept_label (GtkFileChooserNative *self)
 }
 
 /**
- * gtk_file_chooser_native_set_accept_label: (attributes org.gtk.Method.set_property=accept-label)
+ * gtk_file_chooser_native_set_accept_label:
  * @self: a `GtkFileChooserNative`
  * @accept_label: (nullable): custom label
  *
@@ -261,7 +261,7 @@ gtk_file_chooser_native_set_accept_label (GtkFileChooserNative *self,
 }
 
 /**
- * gtk_file_chooser_native_get_cancel_label: (attributes org.gtk.Method.get_property=cancel-label)
+ * gtk_file_chooser_native_get_cancel_label:
  * @self: a `GtkFileChooserNative`
  *
  * Retrieves the custom label text for the cancel button.
@@ -279,7 +279,7 @@ gtk_file_chooser_native_get_cancel_label (GtkFileChooserNative *self)
 }
 
 /**
- * gtk_file_chooser_native_set_cancel_label: (attributes org.gtk.Method.set_property=cancel-label)
+ * gtk_file_chooser_native_set_cancel_label:
  * @self: a `GtkFileChooserNative`
  * @cancel_label: (nullable): custom label
  *
@@ -561,13 +561,6 @@ gtk_file_chooser_native_new (const char           *title,
   return result;
 }
 
-void
-gtk_file_chooser_native_set_use_portal (GtkFileChooserNative *self,
-                                        gboolean              use_portal)
-{
-  self->use_portal = use_portal;
-}
-
 static void
 dialog_response_cb (GtkDialog *dialog,
                     int response_id,
@@ -710,13 +703,6 @@ gtk_file_chooser_native_get_files (GtkFileChooser *chooser)
     }
 }
 
-static void
-portal_error_handler (GtkFileChooserNative *self)
-{
-  self->mode = MODE_FALLBACK;
-  show_dialog (self);
-}
-
 static void
 gtk_file_chooser_native_show (GtkNativeDialog *native)
 {
@@ -735,7 +721,7 @@ gtk_file_chooser_native_show (GtkNativeDialog *native)
 #endif
 
   if (self->mode == MODE_FALLBACK &&
-      gtk_file_chooser_native_portal_show (self, portal_error_handler))
+      gtk_file_chooser_native_portal_show (self))
     self->mode = MODE_PORTAL;
 
   if (self->mode == MODE_FALLBACK)
@@ -786,7 +772,7 @@ gtk_file_chooser_native_class_init (GtkFileChooserNativeClass *class)
   _gtk_file_chooser_install_properties (gobject_class);
 
   /**
-   * GtkFileChooserNative:accept-label: (attributes org.gtk.Property.get=gtk_file_chooser_native_get_accept_label org.gtk.Property.set=gtk_file_chooser_native_set_accept_label)
+   * GtkFileChooserNative:accept-label:
    *
    * The text used for the label on the accept button in the dialog, or
    * %NULL to use the default text.
@@ -797,7 +783,7 @@ gtk_file_chooser_native_class_init (GtkFileChooserNativeClass *class)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkFileChooserNative:cancel-label: (attributes org.gtk.Property.get=gtk_file_chooser_native_get_cancel_label org.gtk.Property.set=gtk_file_chooser_native_set_cancel_label)
+   * GtkFileChooserNative:cancel-label:
    *
    * The text used for the label on the cancel button in the dialog, or
    * %NULL to use the default text.
