@@ -22,12 +22,10 @@
 #include "gtkfilechoosernativeprivate.h"
 #include "gtknativedialogprivate.h"
 
+#include <glib/gi18n-lib.h>
 #include "gtkprivate.h"
-#include "gtkfilechooserdialog.h"
+#include "deprecated/gtkdialog.h"
 #include "gtkfilechooserprivate.h"
-#include "gtkfilechooserwidget.h"
-#include "gtkfilechooserwidgetprivate.h"
-#include "gtkfilechooserutils.h"
 #include "gtksizerequest.h"
 #include "gtktypebuiltins.h"
 #include "gtksettings.h"
@@ -37,8 +35,11 @@
 #include "gtkmain.h"
 #include "gtkfilefilterprivate.h"
 #include "gtkwindowprivate.h"
+#include "gtkalertdialog.h"
 
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 typedef struct {
   GtkFileChooserNative *self;
 
@@ -53,8 +54,8 @@ typedef struct {
 
   const char *method_name;
 
+  char *exported_handle;
   GtkWindow *exported_window;
-  PortalErrorHandler error_handler;
 } FilechooserPortalData;
 
 
@@ -80,7 +81,11 @@ filechooser_portal_data_clear (FilechooserPortalData *data)
 
   if (data->exported_window)
     {
-      gtk_window_unexport_handle (data->exported_window);
+      if (data->exported_handle)
+        {
+          gtk_window_unexport_handle (data->exported_window, data->exported_handle);
+          g_clear_pointer (&data->exported_handle, g_free);
+        }
       g_clear_object (&data->exported_window);
     }
 
@@ -165,12 +170,18 @@ response_cb (GDBusConnection  *connection,
       g_object_unref (filters);
       gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (self), filter_to_select);
       g_object_unref (filter_to_select);
+
+      g_variant_unref (current_filter);
     }
 
   g_slist_free_full (self->custom_files, g_object_unref);
   self->custom_files = NULL;
   for (i = 0; uris[i]; i++)
     self->custom_files = g_slist_prepend (self->custom_files, g_file_new_for_uri (uris[i]));
+  self->custom_files = g_slist_reverse (self->custom_files);
+
+  g_free (uris);
+  g_variant_unref (response_data);
 
   switch (portal_response)
     {
@@ -207,11 +218,9 @@ send_close (FilechooserPortalData *data)
   GError *error = NULL;
 
   message = g_dbus_message_new_method_call (PORTAL_BUS_NAME,
-                                            PORTAL_OBJECT_PATH,
-                                            PORTAL_FILECHOOSER_INTERFACE,
+                                            data->portal_handle,
+                                            PORTAL_REQUEST_INTERFACE,
                                             "Close");
-  g_dbus_message_set_body (message,
-                           g_variant_new ("(o)", data->portal_handle));
 
   if (!g_dbus_connection_send_message (data->connection,
                                        message,
@@ -226,9 +235,9 @@ send_close (FilechooserPortalData *data)
 }
 
 static void
-open_file_msg_cb (GObject *source_object,
+open_file_msg_cb (GObject      *source_object,
                   GAsyncResult *res,
-                  gpointer user_data)
+                  gpointer      user_data)
 {
   FilechooserPortalData *data = user_data;
   GtkFileChooserNative *self = data->self;
@@ -243,12 +252,13 @@ open_file_msg_cb (GObject *source_object,
 
   if (reply == NULL)
     {
-      if (!data->hidden && data->error_handler)
+      if (!data->hidden)
         {
-          data->error_handler (self);
           filechooser_portal_data_free (data);
           self->mode_data = NULL;
         }
+
+      /* FIXME: Show an error dialog here ? */
       g_error_free (error);
       return;
     }
@@ -456,25 +466,45 @@ window_handle_exported (GtkWindow  *window,
       gtk_grab_add (GTK_WIDGET (data->grab_widget));
     }
 
+  data->exported_handle = g_strdup (handle_str);
   show_portal_file_chooser (self, handle_str);
 }
 
 gboolean
-gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
-                                     PortalErrorHandler    error_handler)
+gtk_file_chooser_native_portal_show (GtkFileChooserNative *self)
 {
   FilechooserPortalData *data;
   GtkWindow *transient_for;
   GDBusConnection *connection;
   GtkFileChooserAction action;
   const char *method_name;
+  GdkDisplay *display;
 
-  if (!gdk_should_use_portal ())
+  transient_for = gtk_native_dialog_get_transient_for (GTK_NATIVE_DIALOG (self));
+
+  if (transient_for)
+    display = gtk_widget_get_display (GTK_WIDGET (transient_for));
+  else
+    display = gdk_display_get_default ();
+
+  if (!gdk_display_should_use_portal (display, PORTAL_FILECHOOSER_INTERFACE, 3))
     return FALSE;
 
+  /* From here on out, we want to return TRUE, since we should use the portal,
+   * or fail.
+   */
+
   connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
   if (connection == NULL)
-    return FALSE;
+    {
+      GtkAlertDialog *alert;
+
+      alert = gtk_alert_dialog_new (_("The session bus is not available"));
+      gtk_alert_dialog_show (alert, transient_for);
+      g_object_unref (alert);
+
+      return TRUE;
+    }
 
   action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (self));
 
@@ -483,24 +513,21 @@ gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
   else if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
     method_name = "SaveFile";
   else if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
-    {
-      if (gtk_get_portal_interface_version (connection, "org.freedesktop.portal.FileChooser") < 3)
-        {
-          g_warning ("GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER is not supported by GtkFileChooserNativePortal because portal is too old");
-          return FALSE;
-        }
-      method_name = "OpenFile";
-    }
+    method_name = "OpenFile";
   else
     {
-      g_warning ("GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER is not supported by GtkFileChooserNativePortal");
-      return FALSE;
+      GtkAlertDialog *alert;
+
+      alert = gtk_alert_dialog_new (_("The create-folder action is not supported with portals"));
+      gtk_alert_dialog_show (alert, transient_for);
+      g_object_unref (alert);
+
+      return TRUE;
     }
 
   data = g_new0 (FilechooserPortalData, 1);
   data->self = g_object_ref (self);
   data->connection = connection;
-  data->error_handler = error_handler;
 
   data->method_name = method_name;
 
@@ -509,7 +536,6 @@ gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
 
   self->mode_data = data;
 
-  transient_for = gtk_native_dialog_get_transient_for (GTK_NATIVE_DIALOG (self));
   if (transient_for != NULL && gtk_widget_is_visible (GTK_WIDGET (transient_for)))
     {
       if (!gtk_window_export_handle (transient_for,
@@ -537,8 +563,8 @@ gtk_file_chooser_native_portal_hide (GtkFileChooserNative *self)
 {
   FilechooserPortalData *data = self->mode_data;
 
-  /* This is always set while dialog visible */
-  g_assert (data != NULL);
+  if (data == NULL)
+    return;
 
   data->hidden = TRUE;
 
