@@ -16,8 +16,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_FILE_CHOOSER_NATIVE_PRIVATE_H__
-#define __GTK_FILE_CHOOSER_NATIVE_PRIVATE_H__
+#pragma once
 
 #include <gtk/deprecated/gtkfilechoosernative.h>
 
@@ -52,24 +51,17 @@ struct _GtkFileChooserNative
   GtkWidget *accept_button;
   GtkWidget *cancel_button;
 
-  gboolean use_portal;
   gpointer mode_data;
 };
 
-void gtk_file_chooser_native_set_use_portal (GtkFileChooserNative *self,
-                                             gboolean              use_portal);
-
 gboolean gtk_file_chooser_native_win32_show (GtkFileChooserNative *self);
 void gtk_file_chooser_native_win32_hide (GtkFileChooserNative *self);
 
 gboolean gtk_file_chooser_native_quartz_show (GtkFileChooserNative *self);
 void gtk_file_chooser_native_quartz_hide (GtkFileChooserNative *self);
 
-typedef void (* PortalErrorHandler) (GtkFileChooserNative *self);
-gboolean gtk_file_chooser_native_portal_show (GtkFileChooserNative *self,
-                                              PortalErrorHandler    error_handler);
+gboolean gtk_file_chooser_native_portal_show (GtkFileChooserNative *self);
 void gtk_file_chooser_native_portal_hide (GtkFileChooserNative *self);
 
 G_END_DECLS
 
-#endif /* __GTK_FILE_CHOOSER_NATIVE_PRIVATE_H__ */
