@@ -24,21 +24,23 @@
 #include "gtkdialogerror.h"
 #include "gtkopenuriportal.h"
 #include "deprecated/gtkshow.h"
+#include "gtkprivate.h"
 #include <glib/gi18n-lib.h>
 
+#ifdef G_OS_WIN32
+#include "gtkshowwin32.h"
+#endif
+
 /**
  * GtkFileLauncher:
  *
- * A `GtkFileLauncher` object collects the arguments that are needed to open a
- * file with an application.
+ * Collects the arguments that are needed to open a file with an application.
  *
  * Depending on system configuration, user preferences and available APIs, this
  * may or may not show an app chooser dialog or launch the default application
  * right away.
  *
  * The operation is started with the [method@Gtk.FileLauncher.launch] function.
- * This API follows the GIO async pattern, and the result can be obtained by
- * calling [method@Gtk.FileLauncher.launch_finish].
  *
  * To launch uris that don't represent files, use [class@Gtk.UriLauncher].
  *
@@ -149,7 +151,7 @@ gtk_file_launcher_class_init (GtkFileLauncherClass *class)
   object_class->set_property = gtk_file_launcher_set_property;
 
   /**
-   * GtkFileLauncher:file: (attributes org.gtk.Property.get=gtk_file_launcher_get_file org.gtk.Property.set=gtk_file_launcher_set_file)
+   * GtkFileLauncher:file:
    *
    * The file to launch.
    *
@@ -161,7 +163,7 @@ gtk_file_launcher_class_init (GtkFileLauncherClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFileLauncher:always-ask: (attributes org.gtk.Property.get=gtk_file_launcher_get_always_ask org.gtk.Property.set=gtk_file_launcher_set_always_ask)
+   * GtkFileLauncher:always-ask:
    *
    * Whether to ask the user to choose an app for opening the file. If `FALSE`,
    * the file might be opened with a default app or the previous choice.
@@ -174,7 +176,7 @@ gtk_file_launcher_class_init (GtkFileLauncherClass *class)
                             G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFileLauncher:writable: (attributes org.gtk.Property.get=gtk_file_launcher_get_writable org.gtk.Property.set=gtk_file_launcher_set_writable)
+   * GtkFileLauncher:writable:
    *
    * Whether to make the file writable for the handler.
    *
@@ -214,7 +216,7 @@ gtk_file_launcher_new (GFile *file)
 
 /**
  * gtk_file_launcher_get_file:
- * @self: a `GtkFileLauncher`
+ * @self: a file launcher
  *
  * Gets the file that will be opened.
  *
@@ -232,8 +234,8 @@ gtk_file_launcher_get_file (GtkFileLauncher *self)
 
 /**
  * gtk_file_launcher_set_file:
- * @self: a `GtkFileLauncher`
- * @file: (nullable): a `GFile`
+ * @self: a file launcher
+ * @file: (nullable): the file
  *
  * Sets the file that will be opened.
  *
@@ -254,11 +256,11 @@ gtk_file_launcher_set_file (GtkFileLauncher *self,
 
 /**
  * gtk_file_launcher_get_always_ask:
- * @self: a `GtkFileLauncher`
+ * @self: a file launcher
  *
- * Returns whether to ask the user to choose an app for opening the file.
+ * Returns whether to ask the user which app to use.
  *
- * Returns: `TRUE` if always asking for app
+ * Returns: true if always asking the user
  *
  * Since: 4.12
  */
@@ -272,11 +274,13 @@ gtk_file_launcher_get_always_ask (GtkFileLauncher *self)
 
 /**
  * gtk_file_launcher_set_always_ask:
- * @self: a `GtkFileLauncher`
- * @always_ask: a `gboolean`
+ * @self: a file launcher
+ * @always_ask: whether to always ask
  *
- * Sets whether to awlays ask the user to choose an app for opening the file.
- * If `FALSE`, the file might be opened with a default app or the previous choice.
+ * Sets whether to awlays ask the user which app to use.
+ *
+ * If false, the file might be opened with a default app
+ * or the previous choice.
  *
  * Since: 4.12
  */
@@ -296,11 +300,11 @@ gtk_file_launcher_set_always_ask (GtkFileLauncher *self,
 
 /**
  * gtk_file_launcher_get_writable:
- * @self: a `GtkFileLauncher`
+ * @self: a file launcher
  *
  * Returns whether to make the file writable for the handler.
  *
- * Returns: `TRUE` if the file will be made writable
+ * Returns: true if the file will be made writable
  *
  * Since: 4.14
  */
@@ -314,8 +318,8 @@ gtk_file_launcher_get_writable (GtkFileLauncher *self)
 
 /**
  * gtk_file_launcher_set_writable:
- * @self: a `GtkFileLauncher`
- * @writable: a `gboolean`
+ * @self: a file launcher
+ * @writable: whether to make the file writable
  *
  * Sets whether to make the file writable for the handler.
  *
@@ -434,7 +438,11 @@ show_uri_done (GObject      *source,
   GTask *task = G_TASK (data);
   GError *error = NULL;
 
+#ifndef G_OS_WIN32
   if (!gtk_show_uri_full_finish (parent, result, &error))
+#else
+  if (!gtk_show_uri_win32_finish (parent, result, &error))
+#endif
     {
       if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_task_return_new_error (task, GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_CANCELLED, "Cancelled by user");
@@ -454,20 +462,17 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
 /**
  * gtk_file_launcher_launch:
- * @self: a `GtkFileLauncher`
- * @parent: (nullable): the parent `GtkWindow`
- * @cancellable: (nullable): a `GCancellable` to cancel the operation
- * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @self: a file launcher
+ * @parent: (nullable): the parent window
+ * @cancellable: (nullable): a cancellable to cancel the operation
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
  *
- * Launch an application to open the file.
+ * Launches an application to open the file.
  *
  * This may present an app chooser dialog to the user.
  *
- * The @callback will be called when the operation is completed.
- * It should call [method@Gtk.FileLauncher.launch_finish] to obtain
- * the result.
- *
  * Since: 4.10
  */
 void
@@ -478,6 +483,9 @@ gtk_file_launcher_launch (GtkFileLauncher     *self,
                           gpointer             user_data)
 {
   GTask *task;
+#ifndef G_OS_WIN32
+  GdkDisplay *display;
+#endif
 
   g_return_if_fail (GTK_IS_FILE_LAUNCHER (self));
 
@@ -495,7 +503,12 @@ gtk_file_launcher_launch (GtkFileLauncher     *self,
     }
 
 #ifndef G_OS_WIN32
-  if (gtk_openuri_portal_is_available ())
+  if (parent)
+    display = gtk_widget_get_display (GTK_WIDGET (parent));
+  else
+    display = gdk_display_get_default ();
+
+  if (gdk_display_should_use_portal (display, PORTAL_OPENURI_INTERFACE, 3))
     {
       GtkOpenuriFlags flags = 0;
 
@@ -508,7 +521,6 @@ gtk_file_launcher_launch (GtkFileLauncher     *self,
       gtk_openuri_portal_open_async (self->file, FALSE, flags, parent, cancellable, open_done, task);
     }
   else
-#endif
     {
       char *uri = g_file_get_uri (self->file);
 
@@ -518,19 +530,23 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
       g_free (uri);
     }
+#else /* G_OS_WIN32 */
+  char *path = g_file_get_path (self->file);
+  gtk_show_uri_win32 (parent, path, self->always_ask, cancellable, show_uri_done, task);
+  g_free (path);
+#endif
 }
 
 /**
  * gtk_file_launcher_launch_finish:
- * @self: a `GtkFileLauncher`
- * @result: a `GAsyncResult`
+ * @self: a file launcher
+ * @result: the result
  * @error: return location for a [enum@Gtk.DialogError] or [enum@Gio.Error] error
  *
  * Finishes the [method@Gtk.FileLauncher.launch] call and
  * returns the result.
  *
- * Returns: `TRUE` if an application was launched,
- *     or `FALSE` and @error is set
+ * Returns: true if an application was launched
  *
  * Since: 4.10
  */
@@ -548,21 +564,18 @@ gtk_file_launcher_launch_finish (GtkFileLauncher  *self,
 
 /**
  * gtk_file_launcher_open_containing_folder:
- * @self: a `GtkFileLauncher`
- * @parent: (nullable): the parent `GtkWindow`
- * @cancellable: (nullable): a `GCancellable` to cancel the operation
- * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @self: a file launcher
+ * @parent: (nullable): the parent window
+ * @cancellable: (nullable): a cancellable to cancel the operation
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
  *
- * Launch a file manager to show the file in its parent directory.
+ * Launches a file manager to show the file in its parent directory.
  *
- * This is only supported native files. It will fail if @file
+ * This is only supported for native files. It will fail if @file
  * is e.g. a http:// uri.
  *
- * The @callback will be called when the operation is completed.
- * It should call [method@Gtk.FileLauncher.open_containing_folder_finish]
- * to obtain the result.
- *
  * Since: 4.10
  */
 void
@@ -618,15 +631,14 @@ gtk_file_launcher_open_containing_folder (GtkFileLauncher     *self,
 
 /**
  * gtk_file_launcher_open_containing_folder_finish:
- * @self: a `GtkFileLauncher`
- * @result: a `GAsyncResult`
+ * @self: a file launcher
+ * @result: the result
  * @error: return location for a [enum@Gtk.DialogError] or [enum@Gio.Error] error
  *
  * Finishes the [method@Gtk.FileLauncher.open_containing_folder]
  * call and returns the result.
  *
- * Returns: `TRUE` if an application was launched,
- *     or `FALSE` and @error is set
+ * Returns: true if an application was launched
  *
  * Since: 4.10
  */
@@ -643,4 +655,5 @@ gtk_file_launcher_open_containing_folder_finish (GtkFileLauncher  *self,
 }
 
 /* }}} */
-/* vim:set foldmethod=marker expandtab: */
+
+/* vim:set foldmethod=marker: */
