@@ -37,11 +37,24 @@
  * The URI bound to a `GtkLinkButton` can be set specifically using
  * [method@Gtk.LinkButton.set_uri].
  *
- * By default, `GtkLinkButton` calls [func@Gtk.show_uri] when the button
+ * By default, `GtkLinkButton` calls [method@Gtk.FileLauncher.launch] when the button
  * is clicked. This behaviour can be overridden by connecting to the
  * [signal@Gtk.LinkButton::activate-link] signal and returning %TRUE from
  * the signal handler.
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkLinkButton` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Shift</kbd>+<kbd>F10</kbd> or <kbd>Menu</kbd> opens the context menu.
+ *
+ * # Actions
+ *
+ * `GtkLinkButton` defines a set of built-in actions:
+ *
+ * - `clipboard.copy` copies the url to the clipboard.
+ * - `menu.popup` opens the context menu.
+ *
  * # CSS nodes
  *
  * `GtkLinkButton` has a single CSS node with name button. To differentiate
@@ -57,20 +70,21 @@
 #include "gtklinkbutton.h"
 
 #include "gtkdragsource.h"
+#include "gtkfilelauncher.h"
 #include "gtkgestureclick.h"
 #include "gtkgesturesingle.h"
-#include <glib/gi18n-lib.h>
 #include "gtklabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkpopovermenu.h"
 #include "gtkprivate.h"
-#include "gtkshow.h"
 #include "gtksizerequest.h"
 #include "gtktooltip.h"
+#include "gtkurilauncher.h"
 #include "gtkwidgetprivate.h"
 
 #include <string.h>
+#include <glib/gi18n-lib.h>
 
 typedef struct _GtkLinkButtonClass GtkLinkButtonClass;
 
@@ -169,7 +183,7 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
   klass->activate_link = gtk_link_button_activate_link;
 
   /**
-   * GtkLinkButton:uri: (attributes org.gtk.Property.get=gtk_link_button_get_uri org.gtk.Property.set=gtk_link_button_set_uri)
+   * GtkLinkButton:uri:
    *
    * The URI bound to this button.
    */
@@ -180,7 +194,7 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
                                                         GTK_PARAM_READWRITE));
 
   /**
-   * GtkLinkButton:visited: (attributes org.gtk.Property.get=gtk_link_button_get_visited org.gtk.Property.set=gtk_link_button_set_visited)
+   * GtkLinkButton:visited:
    *
    * The 'visited' state of this button.
    *
@@ -198,7 +212,7 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
    *
    * Emitted each time the `GtkLinkButton` is clicked.
    *
-   * The default handler will call [func@Gtk.show_uri] with the URI
+   * The default handler will call [method@Gtk.FileLauncher.launch] with the URI
    * stored inside the [property@Gtk.LinkButton:uri] property.
    *
    * To override the default behavior, you can connect to the
@@ -215,6 +229,10 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
                   _gtk_boolean_handled_accumulator, NULL,
                   _gtk_marshal_BOOLEAN__VOID,
                   G_TYPE_BOOLEAN, 0);
+  g_signal_set_va_marshaller (link_signals[ACTIVATE_LINK],
+                              G_TYPE_FROM_CLASS (klass),
+                              _gtk_marshal_BOOLEAN__VOIDv);
+
 
   gtk_widget_class_set_css_name (widget_class, I_("button"));
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_LINK);
@@ -230,7 +248,7 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
   /**
    * GtkLinkButton|menu.popup:
    *
-   * Opens the context menu. 
+   * Opens the context menu.
    */
   gtk_widget_class_install_action (widget_class, "menu.popup", NULL, gtk_link_button_popup_menu);
 
@@ -303,7 +321,7 @@ gtk_link_content_get_value (GdkContentProvider  *provider,
       content->link != NULL)
     {
       char *uri;
-  
+
       uri = g_strdup_printf ("%s\r\n", content->link->uri);
       g_value_set_string (value, uri);
       g_free (uri);
@@ -381,7 +399,7 @@ gtk_link_button_get_property (GObject    *object,
 			      GParamSpec *pspec)
 {
   GtkLinkButton *link_button = GTK_LINK_BUTTON (object);
-  
+
   switch (prop_id)
     {
     case PROP_URI:
@@ -403,7 +421,7 @@ gtk_link_button_set_property (GObject      *object,
 			      GParamSpec   *pspec)
 {
   GtkLinkButton *link_button = GTK_LINK_BUTTON (object);
-  
+
   switch (prop_id)
     {
     case PROP_URI:
@@ -475,14 +493,58 @@ gtk_link_button_pressed_cb (GtkGestureClick *gesture,
     }
 }
 
+static void
+launch_done (GObject      *source,
+             GAsyncResult *result,
+             gpointer      data)
+{
+  GError *error = NULL;
+  gboolean success;
+
+  if (GTK_IS_FILE_LAUNCHER (source))
+    success = gtk_file_launcher_launch_finish (GTK_FILE_LAUNCHER (source), result, &error);
+  else if (GTK_IS_URI_LAUNCHER (source))
+    success = gtk_uri_launcher_launch_finish (GTK_URI_LAUNCHER (source), result, &error);
+  else
+    g_assert_not_reached ();
+
+  if (!success)
+    {
+      g_warning ("Failed to launch handler: %s", error->message);
+      g_error_free (error);
+    }
+}
+
 static gboolean
 gtk_link_button_activate_link (GtkLinkButton *link_button)
 {
   GtkWidget *toplevel;
+  const char *uri_scheme;
 
   toplevel = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (link_button)));
 
-  gtk_show_uri (GTK_WINDOW (toplevel), link_button->uri, GDK_CURRENT_TIME);
+  uri_scheme = g_uri_peek_scheme (link_button->uri);
+  if (g_strcmp0 (uri_scheme, "file") == 0)
+    {
+      GFile *file = g_file_new_for_uri (link_button->uri);
+      GtkFileLauncher *launcher;
+
+      launcher = gtk_file_launcher_new (file);
+
+      gtk_file_launcher_launch (launcher, GTK_WINDOW (toplevel), NULL, launch_done, NULL);
+
+      g_object_unref (launcher);
+      g_object_unref (file);
+    }
+  else
+    {
+      GtkUriLauncher *launcher = gtk_uri_launcher_new (link_button->uri);
+
+      gtk_uri_launcher_launch (launcher, GTK_WINDOW (toplevel), NULL, launch_done, NULL);
+
+      g_object_unref (launcher);
+    }
+
   gtk_link_button_set_visited (link_button, TRUE);
 
   return TRUE;
@@ -517,9 +579,9 @@ gtk_link_button_new (const char *uri)
 {
   char *utf8_uri = NULL;
   GtkWidget *retval;
-  
+
   g_return_val_if_fail (uri != NULL, NULL);
-  
+
   if (g_utf8_validate (uri, -1, NULL))
     {
       utf8_uri = g_strdup (uri);
@@ -527,7 +589,7 @@ gtk_link_button_new (const char *uri)
   else
     {
       GError *conv_err = NULL;
-    
+
       utf8_uri = g_locale_to_utf8 (uri, -1, NULL, NULL, &conv_err);
       if (conv_err)
         {
@@ -536,18 +598,18 @@ gtk_link_button_new (const char *uri)
                      uri,
                      conv_err->message);
           g_error_free (conv_err);
-        
+
           utf8_uri = g_strdup (_("Invalid URI"));
         }
     }
-  
+
   retval = g_object_new (GTK_TYPE_LINK_BUTTON,
   			 "label", utf8_uri,
   			 "uri", uri,
   			 NULL);
-  
+
   g_free (utf8_uri);
-  
+
   return retval;
 }
 
@@ -565,9 +627,9 @@ gtk_link_button_new_with_label (const char *uri,
 				const char *label)
 {
   GtkWidget *retval;
-  
+
   g_return_val_if_fail (uri != NULL, NULL);
-  
+
   if (!label)
     return gtk_link_button_new (uri);
 
@@ -579,7 +641,7 @@ gtk_link_button_new_with_label (const char *uri,
   return retval;
 }
 
-static gboolean 
+static gboolean
 gtk_link_button_query_tooltip_cb (GtkWidget    *widget,
                                   int           x,
                                   int           y,
@@ -608,7 +670,7 @@ gtk_link_button_query_tooltip_cb (GtkWidget    *widget,
 }
 
 /**
- * gtk_link_button_set_uri: (attributes org.gtk.Method.set_property=uri)
+ * gtk_link_button_set_uri:
  * @link_button: a `GtkLinkButton`
  * @uri: a valid URI
  *
@@ -632,7 +694,7 @@ gtk_link_button_set_uri (GtkLinkButton *link_button,
 }
 
 /**
- * gtk_link_button_get_uri: (attributes org.gtk.Method.get_property=uri)
+ * gtk_link_button_get_uri:
  * @link_button: a `GtkLinkButton`
  *
  * Retrieves the URI of the `GtkLinkButton`.
@@ -649,7 +711,7 @@ gtk_link_button_get_uri (GtkLinkButton *link_button)
 }
 
 /**
- * gtk_link_button_set_visited: (attributes org.gtk.Method.set_property=visited)
+ * gtk_link_button_set_visited:
  * @link_button: a `GtkLinkButton`
  * @visited: the new “visited” state
  *
@@ -669,6 +731,10 @@ gtk_link_button_set_visited (GtkLinkButton *link_button,
     {
       link_button->visited = visited;
 
+      gtk_accessible_update_state (GTK_ACCESSIBLE (link_button),
+                                   GTK_ACCESSIBLE_STATE_VISITED, visited,
+                                   -1);
+
       if (visited)
         {
           gtk_widget_unset_state_flags (GTK_WIDGET (link_button), GTK_STATE_FLAG_LINK);
@@ -685,7 +751,7 @@ gtk_link_button_set_visited (GtkLinkButton *link_button,
 }
 
 /**
- * gtk_link_button_get_visited: (attributes org.gtk.Method.get_property=visited)
+ * gtk_link_button_get_visited:
  * @link_button: a `GtkLinkButton`
  *
  * Retrieves the “visited” state of the `GtkLinkButton`.
