@@ -1327,10 +1327,15 @@ gtk_list_base_class_init (GtkListBaseClass *klass)
   gtk_list_base_add_custom_move_binding (widget_class, GDK_KEY_Page_Down, gtk_list_base_move_cursor_page_down);
   gtk_list_base_add_custom_move_binding (widget_class, GDK_KEY_KP_Page_Down, gtk_list_base_move_cursor_page_down);
 
+#ifdef __APPLE__
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_a, GDK_META_MASK, "list.select-all", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_A, GDK_META_MASK | GDK_SHIFT_MASK, "list.unselect-all", NULL);
+#else
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_a, GDK_CONTROL_MASK, "list.select-all", NULL);
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_slash, GDK_CONTROL_MASK, "list.select-all", NULL);
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_A, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "list.unselect-all", NULL);
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_backslash, GDK_CONTROL_MASK, "list.unselect-all", NULL);
+#endif
 }
 
 static gboolean
@@ -1702,7 +1707,7 @@ gtk_list_base_apply_rubberband_selection (GtkListBase *self,
   model = gtk_list_item_manager_get_model (priv->item_manager);
   if (model != NULL)
     {
-      GtkBitset *selected, *mask;
+      GtkBitset *selected, *mask, *result;
       GdkRectangle rect;
       GtkBitset *rubberband_selection;
 
@@ -1755,8 +1760,14 @@ gtk_list_base_apply_rubberband_selection (GtkListBase *self,
 
       gtk_selection_model_set_selection (model, selected, mask);
 
+      result = gtk_selection_model_get_selection (model);
+
+      if (gtk_bitset_get_size (result) == 1)
+        gtk_list_base_grab_focus_on_item (self, gtk_bitset_get_minimum (result), TRUE, FALSE, FALSE);
+
       gtk_bitset_unref (selected);
       gtk_bitset_unref (mask);
+      gtk_bitset_unref (result);
       gtk_bitset_unref (rubberband_selection);
     }
 }
@@ -1858,6 +1869,11 @@ get_selection_modifiers (GtkGesture *gesture,
     *modify = TRUE;
   if ((state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK)
     *extend = TRUE;
+
+#ifdef __APPLE__
+  if ((state & GDK_META_MASK) == GDK_META_MASK)
+    *modify = TRUE;
+#endif
 }
 
 static void
