@@ -43,9 +43,7 @@
 #include <sys/types.h>          /* For uid_t, gid_t */
 
 #ifdef G_OS_WIN32
-#define STRICT
 #include <windows.h>
-#undef STRICT
 #endif
 
 #include <hb-glib.h>
@@ -200,6 +198,7 @@ static const GdkDebugKey gtk_debug_keys[] = {
   { "accessibility", GTK_DEBUG_A11Y, "Information about accessibility state changes" },
   { "iconfallback", GTK_DEBUG_ICONFALLBACK, "Information about icon fallback" },
   { "invert-text-dir", GTK_DEBUG_INVERT_TEXT_DIR, "Invert the default text direction" },
+  { "css", GTK_DEBUG_CSS, "Information about deprecated CSS features" },
 };
 
 /* This checks to see if the process is running suid or sgid
@@ -265,6 +264,26 @@ gtk_disable_setlocale (void)
   do_setlocale = FALSE;
 }
 
+/**
+ * gtk_disable_portals:
+ *
+ * Prevents GTK from using portals.
+ *
+ * This is equivalent to setting `GDK_DEBUG=no-portals` in the environment.
+ *
+ * This should only be used in portal implementations, apps must not call it.
+ *
+ * Since: 4.18
+ */
+void
+gtk_disable_portals (void)
+{
+  if (pre_initialized)
+    g_warning ("gtk_disable_portals() must be called before gtk_init()");
+
+  gdk_disable_portals ();
+}
+
 #ifdef G_PLATFORM_WIN32
 #undef gtk_init_check
 #endif
@@ -277,7 +296,7 @@ static char *script_to_check = NULL;
 static gboolean setlocale_called = FALSE;
 
 static BOOL CALLBACK
-enum_locale_proc (LPTSTR locale)
+enum_locale_proc (LPSTR locale)
 {
   LCID lcid;
   char iso639[10];
@@ -287,8 +306,8 @@ enum_locale_proc (LPTSTR locale)
 
   lcid = strtoul (locale, &endptr, 16);
   if (*endptr == '\0' &&
-      GetLocaleInfo (lcid, LOCALE_SISO639LANGNAME, iso639, sizeof (iso639)) &&
-      GetLocaleInfo (lcid, LOCALE_SISO3166CTRYNAME, iso3166, sizeof (iso3166)))
+      GetLocaleInfoA (lcid, LOCALE_SISO639LANGNAME, iso639, sizeof (iso639)) &&
+      GetLocaleInfoA (lcid, LOCALE_SISO3166CTRYNAME, iso3166, sizeof (iso3166)))
     {
       if (strcmp (iso639, iso639_to_check) == 0 &&
           ((iso3166_to_check != NULL &&
@@ -341,8 +360,8 @@ enum_locale_proc (LPTSTR locale)
 
           SetThreadLocale (lcid);
 
-          if (GetLocaleInfo (lcid, LOCALE_SENGLANGUAGE, language, sizeof (language)) &&
-              GetLocaleInfo (lcid, LOCALE_SENGCOUNTRY, country, sizeof (country)))
+          if (GetLocaleInfoA (lcid, LOCALE_SENGLANGUAGE, language, sizeof (language)) &&
+              GetLocaleInfoA (lcid, LOCALE_SENGCOUNTRY, country, sizeof (country)))
             {
               char str[300];
 
@@ -424,7 +443,7 @@ setlocale_initialization (void)
                     iso3166_to_check = (char *) "SP";
                 }
 
-              EnumSystemLocales (enum_locale_proc, LCID_SUPPORTED);
+              EnumSystemLocalesA (enum_locale_proc, LCID_SUPPORTED);
             }
           g_free (p);
         }
@@ -482,8 +501,11 @@ do_pre_parse_initialization (void)
   gdk_pre_parse ();
 
   debug_flags[0].flags = gdk_parse_debug_var ("GTK_DEBUG",
-                                              gtk_debug_keys,
-                                              G_N_ELEMENTS (gtk_debug_keys));
+      "GTK_DEBUG can be set to values that make GTK print out different\n"
+      "types of debugging information or change the behavior of GTK for\n"
+      "debugging purposes.\n",
+      gtk_debug_keys,
+      G_N_ELEMENTS (gtk_debug_keys));
   any_display_debug_flags_set = debug_flags[0].flags > 0;
 
   env_string = g_getenv ("GTK_SLOWDOWN");
@@ -627,18 +649,19 @@ gtk_init_check (void)
  * gtk_init:
  *
  * Call this function before using any other GTK functions in your GUI
- * applications. It will initialize everything needed to operate the
- * toolkit.
+ * applications.
+ *
+ * It will initialize everything needed to operate the toolkit. In particular,
+ * it will open the default display (see [func@Gdk.Display.get_default]).
  *
  * If you are using `GtkApplication`, you usually don't have to call this
  * function; the `GApplication::startup` handler does it for you. Though,
  * if you are using GApplication methods that will be invoked before `startup`,
  * such as `local_command_line`, you may need to initialize stuff explicitly.
  *
- * This function will terminate your program if it was unable to
- * initialize the windowing system for some reason. If you want
- * your program to fall back to a textual interface, call
- * [func@Gtk.init_check] instead.
+ * This function will terminate your program if it was unable to initialize
+ * the windowing system for some reason. If you want your program to fall back
+ * to a textual interface, call [func@Gtk.init_check] instead.
  *
  * GTK calls `signal (SIGPIPE, SIG_IGN)` during initialization, to ignore
  * SIGPIPE signals, since these are almost never wanted in graphical
@@ -1925,7 +1948,7 @@ gtk_propagate_event_internal (GtkWidget *widget,
       i--;
     }
 
-  /* If not yet handled, also propagate down */
+  /* If not yet handled, also propagate back up */
   if (!handled_event)
     {
       /* Propagate event up the widget tree so that
