@@ -142,7 +142,7 @@ gtk_print_setup_unref (GtkPrintSetup *setup)
  * They may be different from the `GtkPrintDialog`'s settings
  * if the user changed them during the setup process.
  *
- * Returns: (nullable) (transfer none): the print settings, or `NULL`
+ * Returns: (transfer none): the print settings, or `NULL`
  *
  * Since: 4.14
  */
@@ -168,7 +168,7 @@ gtk_print_setup_set_print_settings (GtkPrintSetup    *setup,
  * It may be different from the `GtkPrintDialog`'s page setup
  * if the user changed it during the setup process.
  *
- * Returns: (nullable) (transfer none): the page setup, or `NULL`
+ * Returns: (transfer none): the page setup, or `NULL`
  *
  * Since: 4.14
  */
@@ -188,6 +188,7 @@ gtk_print_setup_set_page_setup (GtkPrintSetup *setup,
 static GtkPrinter *
 gtk_print_setup_get_printer (GtkPrintSetup *setup)
 {
+#ifdef HAVE_GIO_UNIX
   if (!setup->printer)
     {
       const char *name = NULL;
@@ -198,6 +199,7 @@ gtk_print_setup_get_printer (GtkPrintSetup *setup)
       if (name)
         setup->printer = gtk_printer_find (name);
     }
+#endif
 
   return setup->printer;
 }
@@ -359,7 +361,7 @@ gtk_print_dialog_class_init (GtkPrintDialogClass *class)
   object_class->set_property = gtk_print_dialog_set_property;
 
   /**
-   * GtkPrintDialog:accept-label: (attributes org.gtk.Property.get=gtk_print_dialog_get_accept_label org.gtk.Property.set=gtk_print_dialog_set_accept_label)
+   * GtkPrintDialog:accept-label:
    *
    * A label that may be shown on the accept button of a print dialog
    * that is presented by [method@Gtk.PrintDialog.setup].
@@ -372,7 +374,7 @@ gtk_print_dialog_class_init (GtkPrintDialogClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPrintDialog:page-setup: (attributes org.gtk.Property.get=gtk_print_dialog_get_page_setup org.gtk.Property.set=gtk_print_dialog_set_page_setup)
+   * GtkPrintDialog:page-setup:
    *
    * The page setup to use.
    *
@@ -384,7 +386,7 @@ gtk_print_dialog_class_init (GtkPrintDialogClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPrintDialog:modal: (attributes org.gtk.Property.get=gtk_print_dialog_get_modal org.gtk.Property.set=gtk_print_dialog_set_modal)
+   * GtkPrintDialog:modal:
    *
    * Whether the print dialog is modal.
    *
@@ -396,7 +398,7 @@ gtk_print_dialog_class_init (GtkPrintDialogClass *class)
                             G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPrintDialog:print-settings: (attributes org.gtk.Property.get=gtk_print_dialog_get_print_settings org.gtk.Property.set=gtk_print_dialog_set_print_settings)
+   * GtkPrintDialog:print-settings:
    *
    * The print settings to use.
    *
@@ -408,7 +410,7 @@ gtk_print_dialog_class_init (GtkPrintDialogClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkPrintDialog:title: (attributes org.gtk.Property.get=gtk_print_dialog_get_title org.gtk.Property.set=gtk_print_dialog_set_title)
+   * GtkPrintDialog:title:
    *
    * A title that may be shown on the print dialog that is
    * presented by [method@Gtk.PrintDialog.setup].
@@ -585,7 +587,7 @@ gtk_print_dialog_set_modal (GtkPrintDialog *self,
  *
  * Returns the page setup.
  *
- * Returns: (transfer none): the page setup
+ * Returns: (nullable) (transfer none): the page setup
  *
  * Since: 4.14
  */
@@ -623,7 +625,7 @@ gtk_print_dialog_set_page_setup (GtkPrintDialog *self,
  *
  * Returns the print settings for the print dialog.
  *
- * Returns: (transfer none): the settings
+ * Returns: (nullable) (transfer none): the settings
  *
  * Since: 4.14
  */
@@ -752,6 +754,7 @@ gtk_print_output_stream_set_print_done (GtkPrintOutputStream *stream,
 typedef struct
 {
   GtkWindow *exported_window;
+  char *exported_window_handle;
   char *portal_handle;
   unsigned int response_signal_id;
   unsigned int token;
@@ -776,6 +779,9 @@ print_task_data_free (gpointer data)
   PrintTaskData *ptd = data;
 
   g_free (ptd->portal_handle);
+  if (ptd->exported_window && ptd->exported_window_handle)
+    gtk_window_unexport_handle (ptd->exported_window, ptd->exported_window_handle);
+  g_clear_pointer (&ptd->exported_window_handle, g_free);
   g_clear_object (&ptd->exported_window);
   if (ptd->fds[0] != -1)
     close (ptd->fds[0]);
@@ -817,10 +823,11 @@ send_close (GTask *task)
 
 static gboolean
 ensure_portal_proxy (GtkPrintDialog  *self,
+                     GtkWindow       *parent,
                      GError         **error)
 {
-  if (gdk_display_get_debug_flags (NULL) & GDK_DEBUG_NO_PORTALS)
-    return FALSE;
+  if (self->portal)
+    return TRUE;
 
   if (!self->portal)
     self->portal = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
@@ -870,8 +877,12 @@ cleanup_portal_call_data (GTask *task)
       ptd->response_signal_id = 0;
     }
 
+  if (ptd->exported_window && ptd->exported_window_handle)
+    gtk_window_unexport_handle (ptd->exported_window, ptd->exported_window_handle);
+
   g_clear_pointer (&ptd->portal_handle, g_free);
   g_clear_object (&ptd->exported_window);
+  g_clear_pointer (&ptd->exported_window_handle, g_free);
 }
 
 /* }}} */
@@ -1014,6 +1025,12 @@ setup_window_handle_exported (GtkWindow  *window,
   g_assert (ptd->portal_handle == NULL);
   ptd->portal_handle = gtk_get_portal_request_path (connection, &handle_token);
 
+  if (window)
+    {
+      ptd->exported_window = g_object_ref (window);
+      ptd->exported_window_handle = g_strdup (window_handle);
+    }
+
   g_assert (ptd->response_signal_id == 0);
   ptd->response_signal_id =
     g_dbus_connection_signal_subscribe (connection,
@@ -1211,7 +1228,10 @@ print_window_handle_exported (GtkWindow  *window,
   int idx;
 
   if (window)
-    ptd->exported_window = g_object_ref (window);
+    {
+      ptd->exported_window = g_object_ref (window);
+      ptd->exported_window_handle = g_strdup (window_handle);
+    }
 
   g_assert (ptd->fds[0] != -1);
 
@@ -1423,17 +1443,16 @@ print_response_cb (GtkPrintUnixDialog *window,
  * @self: a `GtkPrintDialog`
  * @parent: (nullable): the parent `GtkWindow`
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
- * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
  *
  * This function presents a print dialog to let the user select a printer,
  * and set up print settings and page setup.
  *
  * The @callback will be called when the dialog is dismissed.
- * It should call [method@Gtk.PrintDialog.setup_finish]
- * to obtain the results in the form of a [struct@Gtk.PrintSetup],
- * that can then be passed to [method@Gtk.PrintDialog.print]
- * or [method@Gtk.PrintDialog.print_file].
+ * The obtained [struct@Gtk.PrintSetup] can then be passed
+ * to [method@Gtk.PrintDialog.print] or [method@Gtk.PrintDialog.print_file].
  *
  * One possible use for this method is to have the user select a printer,
  * then show a page setup UI in the application (e.g. to arrange images
@@ -1450,6 +1469,9 @@ gtk_print_dialog_setup (GtkPrintDialog       *self,
                         gpointer              user_data)
 {
   GTask *task;
+#ifdef HAVE_GIO_UNIX
+  GdkDisplay *display;
+#endif
   G_GNUC_UNUSED GError *error = NULL;
 
   g_return_if_fail (GTK_IS_PRINT_DIALOG (self));
@@ -1463,7 +1485,12 @@ gtk_print_dialog_setup (GtkPrintDialog       *self,
   if (cancellable)
     g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
 
-  if (!ensure_portal_proxy (self, &error))
+  if (parent)
+    display = gtk_widget_get_display (GTK_WIDGET (parent));
+  else
+    display = gdk_display_get_default ();
+
+  if (!gdk_display_should_use_portal (display, PORTAL_PRINT_INTERFACE, 0))
     {
       GtkPrintUnixDialog *window;
 
@@ -1471,6 +1498,14 @@ gtk_print_dialog_setup (GtkPrintDialog       *self,
       g_signal_connect (window, "response", G_CALLBACK (setup_response_cb), task);
       gtk_window_present (GTK_WINDOW (window));
     }
+  else if (!ensure_portal_proxy (self, parent, &error))
+    {
+      g_task_return_new_error (task,
+                               GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_FAILED,
+                               "The print portal is not available: %s", error->message);
+      g_error_free (error);
+      g_object_unref (task);
+    }
   else
     {
       g_task_set_task_data (task, print_task_data_new (), (GDestroyNotify) print_task_data_free);
@@ -1502,7 +1537,7 @@ gtk_print_dialog_setup (GtkPrintDialog       *self,
  * which contains the print settings and page setup information that
  * will be used to print.
  *
- * Returns: (nullable): The `GtkPrintSetup` object that resulted from the call,
+ * Returns: The `GtkPrintSetup` object that resulted from the call,
  *   or `NULL` if the call was not successful
  *
  * Since: 4.14
@@ -1525,16 +1560,16 @@ gtk_print_dialog_setup_finish (GtkPrintDialog    *self,
  * @parent: (nullable): the parent `GtkWindow`
  * @setup: (nullable): the `GtkPrintSetup` to use
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
- * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
  *
  * This function prints content from a stream.
  *
  * If you pass `NULL` as @setup, then this method will present a print dialog.
  * Otherwise, it will attempt to print directly, without user interaction.
  *
- * The @callback will be called when the printing is done. It should call
- * [method@Gtk.PrintDialog.print_finish] to obtain the results.
+ * The @callback will be called when the printing is done.
  *
  * Since: 4.14
  */
@@ -1549,6 +1584,7 @@ gtk_print_dialog_print (GtkPrintDialog       *self,
   GTask *task;
   G_GNUC_UNUSED GError *error = NULL;
 #ifdef HAVE_GIO_UNIX
+  GdkDisplay *display;
   PrintTaskData *ptd;
 #endif
 
@@ -1574,7 +1610,12 @@ gtk_print_dialog_print (GtkPrintDialog       *self,
   if (cancellable)
     g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
 
-  if (!ensure_portal_proxy (self, &error))
+  if (parent)
+    display = gtk_widget_get_display (GTK_WIDGET (parent));
+  else
+    display = gdk_display_get_default ();
+
+  if (!gdk_display_should_use_portal (display, PORTAL_PRINT_INTERFACE, 0))
     {
       if (setup == NULL || gtk_print_setup_get_printer (setup) == NULL)
         {
@@ -1592,6 +1633,14 @@ gtk_print_dialog_print (GtkPrintDialog       *self,
           print_content (setup, task);
         }
     }
+  else if (!ensure_portal_proxy (self, parent, &error))
+    {
+      g_task_return_new_error (task,
+                               GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_FAILED,
+                               "The print portal is not available: %s", error->message);
+      g_error_free (error);
+      g_object_unref (task);
+    }
   else
     {
       if (parent &&
@@ -1628,7 +1677,7 @@ gtk_print_dialog_print (GtkPrintDialog       *self,
  * call may not be instant as it operation will for the printer to finish
  * printing.
  *
- * Returns: (nullable) (transfer full): a [class@Gio.OutputStream]
+ * Returns: (transfer full): a [class@Gio.OutputStream]
  *
  * Since: 4.14
  */
@@ -1651,17 +1700,15 @@ gtk_print_dialog_print_finish (GtkPrintDialog  *self,
  * @setup: (nullable):  the `GtkPrintSetup` to use
  * @file: the `GFile` to print
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
- * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @callback: (scope async) (closure user_data): a callback to call when the
+ *   operation is complete
+ * @user_data: data to pass to @callback
  *
  * This function prints a file.
  *
  * If you pass `NULL` as @setup, then this method will present a print dialog.
  * Otherwise, it will attempt to print directly, without user interaction.
  *
- * The @callback will be called when the printing is done. It should call
- * [method@Gtk.PrintDialog.print_file_finish] to obtain the results.
- *
  * Since: 4.14
  */
 void
@@ -1675,10 +1722,11 @@ gtk_print_dialog_print_file (GtkPrintDialog       *self,
 {
   GTask *task;
 #ifdef HAVE_GIO_UNIX
+  GdkDisplay *display;
   PrintTaskData *ptd;
   GFileInputStream *content;
-#endif
   GError *error = NULL;
+#endif
 
   g_return_if_fail (GTK_IS_PRINT_DIALOG (self));
   g_return_if_fail (parent == NULL || GTK_IS_WINDOW (parent));
@@ -1702,7 +1750,7 @@ gtk_print_dialog_print_file (GtkPrintDialog       *self,
     {
       g_task_return_new_error (task,
                                GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_FAILED,
-                               "Failed to create read fd");
+                               _("Failed to create the read file descriptor"));
       g_object_unref (task);
       return;
     }
@@ -1710,7 +1758,12 @@ gtk_print_dialog_print_file (GtkPrintDialog       *self,
  if (cancellable)
     g_signal_connect (cancellable, "cancelled", G_CALLBACK (cancelled_cb), task);
 
-  if (!ensure_portal_proxy (self, &error))
+  if (parent)
+    display = gtk_widget_get_display (GTK_WIDGET (parent));
+  else
+    display = gdk_display_get_default ();
+
+  if (!gdk_display_should_use_portal (display, PORTAL_PRINT_INTERFACE, 0))
     {
       if (setup == NULL || gtk_print_setup_get_printer (setup) == NULL)
         {
@@ -1728,6 +1781,14 @@ gtk_print_dialog_print_file (GtkPrintDialog       *self,
           print_content (setup, task);
         }
     }
+  else if (!ensure_portal_proxy (self, parent, &error))
+    {
+      g_task_return_new_error (task,
+                               GTK_DIALOG_ERROR, GTK_DIALOG_ERROR_FAILED,
+                               "The print portal is not available: %s", error->message);
+      g_error_free (error);
+      g_object_unref (task);
+    }
   else
     {
       if (parent &&
@@ -1772,4 +1833,4 @@ gtk_print_dialog_print_file_finish (GtkPrintDialog  *self,
 
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
