@@ -84,7 +84,7 @@ fetch_request_mode (GtkWidget *widget)
 static int
 get_number (GtkCssValue *value)
 {
-  double d = _gtk_css_number_value_get (value, 100);
+  double d = gtk_css_number_value_get (value, 100);
 
   if (d < 1)
     return ceil (d);
@@ -96,7 +96,7 @@ get_number (GtkCssValue *value)
 static int
 get_number_ceil (GtkCssValue *value)
 {
-  return ceil (_gtk_css_number_value_get (value, 100));
+  return ceil (gtk_css_number_value_get (value, 100));
 }
 
 static void
@@ -493,7 +493,13 @@ gtk_widget_measure (GtkWidget        *widget,
       int min_opposite_size;
       gtk_widget_measure (widget, OPPOSITE_ORIENTATION (orientation), -1, &min_opposite_size, NULL, NULL, NULL);
       if (for_size < min_opposite_size)
-        for_size = min_opposite_size;
+        {
+          g_warning ("Trying to measure %s %p for %s of %d, but it needs at least %d",
+                     gtk_widget_get_name (widget), widget,
+                     (orientation == GTK_ORIENTATION_VERTICAL) ? "width" : "height",
+                     for_size, min_opposite_size);
+          for_size = min_opposite_size;
+        }
     }
 
   /* This is the main function that checks for a cached size and
